# coding: utf-8
# copyright 2021-2022 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

from rdflib import BNode, Literal, URIRef
from rdflib.term import _is_valid_uri

from logilab.common.decorators import cachedproperty

from cubicweb.predicates import is_instance
from cubicweb.entities.adapters import EntityRDFAdapter
from cubicweb.uilib import remove_html_tags


class AuthorityRecordRDFAdapter(EntityRDFAdapter):
    __regid__ = "rdf.schemaorg"
    __select__ = EntityRDFAdapter.__select__ & is_instance("AuthorityRecord")

    def names_triples(self):
        SCHEMA = self._use_namespace("schema")
        entity = self.entity
        for name_entry in entity.reverse_name_entry_for:
            if name_entry.parts:
                yield (self.uri, SCHEMA.name, Literal(name_entry.parts))

    def person_triples(self):
        SCHEMA = self._use_namespace("schema")
        RDF = self._use_namespace("rdf")
        entity = self.entity
        yield (self.uri, RDF.type, SCHEMA.Person)
        if entity.start_date:
            yield (
                self.uri,
                SCHEMA.birthDate,
                Literal(entity.start_date.strftime("%Y-%m-%d")),
            )
        if entity.end_date:
            yield (
                self.uri,
                SCHEMA.deathDate,
                Literal(entity.end_date.strftime("%Y-%m-%d")),
            )
        if entity.reverse_occupation_agent:
            yield (
                self.uri,
                SCHEMA.hasOccupation,
                Literal(entity.reverse_occupation_agent[0].term),
            )
        if entity.reverse_history_agent:
            yield (
                self.uri,
                SCHEMA.description,
                Literal(remove_html_tags(entity.reverse_history_agent[0].text)),
            )
        for family_member in entity.reverse_family_from:
            yield (self.uri, SCHEMA.relatedTo, Literal(family_member.entry))

    def organization_triples(self):
        SCHEMA = self._use_namespace("schema")
        RDF = self._use_namespace("rdf")
        entity = self.entity
        yield (self.uri, RDF.type, SCHEMA.Organization)
        if entity.start_date:
            yield (
                self.uri,
                SCHEMA.foundingDate,
                Literal(entity.start_date.strftime("%Y-%m-%d")),
            )
        if entity.end_date:
            yield (
                self.uri,
                SCHEMA.dissolutionDate,
                Literal(entity.end_date.strftime("%Y-%m-%d")),
            )

        for parent_of_relation in entity.reverse_hierarchical_parent:
            yield (
                self.uri,
                SCHEMA.subOrganization,
                Literal(parent_of_relation.hierarchical_child[0].dc_title()),
            )

        for child_of_relation in entity.reverse_hierarchical_child:
            yield (
                self.uri,
                SCHEMA.parentOrganization,
                Literal(child_of_relation.hierarchical_parent[0].dc_title()),
            )

    def triples(self):
        SCHEMA = self._use_namespace("schema")
        entity = self.entity
        yield (self.uri, SCHEMA.url, Literal(self.uri))
        yield from self.names_triples()

        for identity_relation in entity.reverse_identity_from:
            same_as_entity = identity_relation.identity_to[0]
            if same_as_entity.cw_etype == "ExternalUri":
                yield (self.uri, SCHEMA.sameAs, Literal(same_as_entity.uri))
            else:
                same_as_uri = same_as_entity.cw_adapt_to("rdf").uri
                yield (self.uri, SCHEMA.sameAs, Literal(same_as_uri))
        if entity.agent_kind:
            if entity.agent_kind[0].name == "person":
                yield from self.person_triples()
            else:
                yield from self.organization_triples()


def directional_relation_uri(relation_type, source_record, target_record):
    return URIRef(
        f"{source_record.cw_adapt_to('rdf').uri}#{relation_type}_to_{target_record.record_id}"
    )


def symmetric_relation_uri(relation_type, record_a, record_b):
    records = sorted([record_a, record_b], key=lambda x: x.record_id)
    return URIRef(
        f"{records[0].cw_adapt_to('rdf').uri}#{relation_type}_to_{records[1].record_id}"
    )


class AuthorityRecordRICORDFAdapter(EntityRDFAdapter):
    __regid__ = "rdf.rico"
    __select__ = EntityRDFAdapter.__select__ & is_instance("AuthorityRecord")

    @cachedproperty
    def agent_uri(self):
        return URIRef(f"{self.uri}#agent")

    @cachedproperty
    def inst_uri(self):
        return URIRef(f"{self.uri}#inst")

    def relation_triples(
        self,
        class_uri,
        relation_uri,
        source_uri,
        target_uri,
        prop_source,
        prop_source_inverse,
        prop_target,
        prop_target_inverse,
        reduced_prop,
        reduced_prop_inverse,
        desc=None,
        start=None,
        end=None,
    ):
        RDF = self._use_namespace("rdf")
        RICO = self._use_namespace(
            "rico", base_url="https://www.ica.org/standards/RiC/ontology#"
        )
        yield (relation_uri, RDF.type, class_uri)
        yield (relation_uri, prop_source, source_uri)
        yield (relation_uri, prop_target, target_uri)
        yield (source_uri, prop_source_inverse, relation_uri)
        yield (target_uri, prop_target_inverse, relation_uri)
        yield (source_uri, reduced_prop, target_uri)
        yield (target_uri, reduced_prop_inverse, source_uri)
        if start:
            yield (relation_uri, RICO.beginningDate, Literal(start))
        if end:
            yield (relation_uri, RICO.endDate, Literal(end))
        if desc:
            yield (relation_uri, RICO.descriptiveNote, Literal(remove_html_tags(desc)))

    def person_triples(self):
        RICO = self._use_namespace(
            "rico", base_url="https://www.ica.org/standards/RiC/ontology#"
        )
        if self.entity.start_date:
            yield (self.agent_uri, RICO.birthDate, Literal(self.entity.start_date))
        if self.entity.end_date:
            yield (self.agent_uri, RICO.deathDate, Literal(self.entity.end_date))
        family_relations = self._cw.execute(
            "Any AGENT, KIND, DESC WHERE "
            "F is FamilyRelation, F family_from X,"
            "F family_to AGENT, AGENT agent_kind K, K name KIND,"
            "AGENT is AuthorityRecord, F description DESC,"
            "X is AuthorityRecord, X eid %(eid)s",
            {"eid": self.entity.eid},
        )
        if family_relations:
            for agent_eid, kind, desc in family_relations:
                fam_agent_record = self._cw.entity_from_eid(agent_eid)
                relation_uri = symmetric_relation_uri(
                    "family", fam_agent_record, self.entity
                )

                fam_agent_uri = fam_agent_record.cw_adapt_to(self.__regid__).agent_uri

                if kind == "person":
                    yield from self.relation_triples(
                        RICO.FamilyRelation,
                        relation_uri,
                        self.agent_uri,
                        fam_agent_uri,
                        RICO.familyRelationConnects,
                        RICO.personHasFamilyRelation,
                        RICO.familyRelationConnects,
                        RICO.personHasFamilyRelation,
                        RICO.hasFamilyAssociationWith,
                        RICO.hasFamilyAssociationWith,
                        desc=desc,
                    )
                else:
                    yield from self.relation_triples(
                        RICO.MembershipRelation,
                        relation_uri,
                        fam_agent_uri,
                        self.agent_uri,
                        RICO.membershipRelationHasSource,
                        RICO.groupIsSourceOfMembershipRelation,
                        RICO.membershipRelationHasTarget,
                        RICO.personIsTargetOfMembershipRelation,
                        RICO.hasOrHadMember,
                        RICO.isOrWasMemberOf,
                        desc=desc,
                    )

    def family_triples(self):
        RICO = self._use_namespace(
            "rico", base_url="https://www.ica.org/standards/RiC/ontology#"
        )
        if self.entity.start_date:
            yield (self.agent_uri, RICO.beginningDate, Literal(self.entity.start_date))
        if self.entity.end_date:
            yield (self.agent_uri, RICO.endDate, Literal(self.entity.end_date))

        family_relations = self._cw.execute(
            "Any F, AGENT, KIND, DESC WHERE "
            "F is FamilyRelation, F family_from X,"
            "F family_to AGENT, AGENT agent_kind K, K name KIND,"
            "AGENT is AuthorityRecord, F description DESC,"
            "X is AuthorityRecord, X eid %(eid)s",
            {"eid": self.entity.eid},
        )
        if family_relations:
            for rel_eid, agent_eid, kind, desc in family_relations:
                fam_agent_record = self._cw.entity_from_eid(agent_eid)
                relation_uri = symmetric_relation_uri(
                    "family", fam_agent_record, self.entity
                )

                fam_agent_uri = fam_agent_record.cw_adapt_to(self.__regid__).agent_uri
                if kind == "person":
                    yield from self.relation_triples(
                        RICO.MembershipRelation,
                        relation_uri,
                        self.agent_uri,
                        fam_agent_uri,
                        RICO.membershipRelationHasSource,
                        RICO.groupIsSourceOfMembershipRelation,
                        RICO.membershipRelationHasTarget,
                        RICO.personIsTargetOfMembershipRelation,
                        RICO.hasOrHadMember,
                        RICO.isOrWasMemberOf,
                        desc=desc,
                    )
                else:
                    yield from self.relation_triples(
                        RICO.AgentToAgentRelation,
                        relation_uri,
                        self.agent_uri,
                        fam_agent_uri,
                        RICO.agentRelationConnects,
                        RICO.agentIsConnectedToAgentRelation,
                        RICO.agentRelationConnects,
                        RICO.agentIsConnectedToAgentRelation,
                        RICO.isAgentAssociatedWithAgent,
                        RICO.isAgentAssociatedWithAgent,
                    )

    def hierarchical_triples(
        self, relation_uri, parent_uri, child_uri, desc, start, end
    ):
        RICO = self._use_namespace(
            "rico", base_url="https://www.ica.org/standards/RiC/ontology#"
        )
        yield from self.relation_triples(
            RICO.AgentHierarchicalRelation,
            relation_uri,
            parent_uri,
            child_uri,
            RICO.agentHierarchicalRelationHasSource,
            RICO.agentIsSourceOfAgentHierarchicalRelation,
            RICO.agentHierarchicalRelationHasTarget,
            RICO.agentIsTargetOfAgentHierarchicalRelation,
            RICO.hasOrHadSubordinate,
            RICO.isOrWasSubordinateTo,
            desc,
            start,
            end,
        )

    def successor_triples(
        self, relation_uri, predecessor_uri, successor_uri, desc, start
    ):
        RICO = self._use_namespace(
            "rico", base_url="https://www.ica.org/standards/RiC/ontology#"
        )
        yield from self.relation_triples(
            RICO.AgentTemporalRelation,
            relation_uri,
            predecessor_uri,
            successor_uri,
            RICO.agentTemporalRelationHasSource,
            RICO.agentIsSourceOfAgentTemporalRelation,
            RICO.agentTemporalRelationHasTarget,
            RICO.agentIsTargetOfAgentTemporalRelation,
            RICO.hasSuccessor,
            RICO.isSuccessorOf,
            desc=desc,
        )
        if start:
            yield (relation_uri, RICO.date, Literal(start))

    def authority_triples(self):
        RICO = self._use_namespace(
            "rico", base_url="https://www.ica.org/standards/RiC/ontology#"
        )
        if self.entity.start_date:
            yield (
                self.agent_uri,
                RICO.beginningDate,
                Literal(self.entity.start_date),
            )
        if self.entity.end_date:
            yield (self.agent_uri, RICO.endDate, Literal(self.entity.end_date))

        sub_organizations = self._cw.execute(
            "Any AGENT, DESC, START, END WHERE "
            "R is HierarchicalRelation, R hierarchical_parent X,"
            "R hierarchical_child AGENT,"
            "AGENT is AuthorityRecord, R description DESC,"
            "X is AuthorityRecord, X eid %(eid)s,"
            "R date_relation D?, D is DateEntity,"
            "D start_date START?, D end_date END?",
            {"eid": self.entity.eid},
        )
        if sub_organizations:
            for agent_eid, desc, start, end in sub_organizations:
                child_authority_record = self._cw.entity_from_eid(agent_eid)
                relation_uri = directional_relation_uri(
                    "hierarchical", self.entity, child_authority_record
                )

                child_uri = child_authority_record.cw_adapt_to(self.__regid__).agent_uri

                yield from self.hierarchical_triples(
                    relation_uri, self.agent_uri, child_uri, desc, start, end
                )
        super_organizations = self._cw.execute(
            "Any AGENT, DESC, START, END WHERE "
            "R is HierarchicalRelation, R hierarchical_child X,"
            "R hierarchical_parent AGENT,"
            "AGENT is AuthorityRecord, R description DESC,"
            "X is AuthorityRecord, X eid %(eid)s,"
            "R date_relation D?, D is DateEntity,"
            "D start_date START?, D end_date END?",
            {"eid": self.entity.eid},
        )
        if super_organizations:
            for agent_eid, desc, start, end in super_organizations:
                parent_authority_record = self._cw.entity_from_eid(agent_eid)
                relation_uri = directional_relation_uri(
                    "hierarchical", parent_authority_record, self.entity
                )

                parent_uri = parent_authority_record.cw_adapt_to(
                    self.__regid__
                ).agent_uri

                yield from self.hierarchical_triples(
                    relation_uri, parent_uri, self.agent_uri, desc, start, end
                )

    def agent_triples(self):
        RICO = self._use_namespace(
            "rico", base_url="https://www.ica.org/standards/RiC/ontology#"
        )
        RDF = self._use_namespace("rdf")
        RDFS = self._use_namespace("rdfs")
        OWL = self._use_namespace("owl")

        KIND_TO_URI = {
            "person": RICO.Person,
            "authority": RICO.CorporateBody,
            "family": RICO.Family,
        }

        yield (self.agent_uri, RDF.type, RICO.Agent)
        agent_kind = self.entity.agent_kind[0].name
        if agent_kind in KIND_TO_URI:
            yield (
                self.agent_uri,
                RDF.type,
                KIND_TO_URI[self.entity.agent_kind[0].name],
            )

        if agent_kind == "person":
            yield from self.person_triples()
        elif agent_kind == "family":
            yield from self.family_triples()
        else:
            yield from self.authority_triples()

        association_relations = self._cw.execute(
            "Any R, AGENT, DESC, START, END WHERE "
            "R is AssociationRelation, R association_from X,"
            "R association_to AGENT, R description DESC,"
            "AGENT is AuthorityRecord,"
            "X is AuthorityRecord, X eid %(eid)s,"
            "R date_relation D?, D is DateEntity,"
            "D start_date START?, D end_date END?",
            {"eid": self.entity.eid},
        )
        if association_relations:
            for rel_eid, agent_eid, desc, start, end in association_relations:
                associated_agent_record = self._cw.entity_from_eid(agent_eid)
                relation_uri = symmetric_relation_uri(
                    "association", associated_agent_record, self.entity
                )

                rel_agent_uri = associated_agent_record.cw_adapt_to(
                    self.__regid__
                ).agent_uri
                yield from self.relation_triples(
                    RICO.AgentToAgentRelation,
                    relation_uri,
                    self.agent_uri,
                    rel_agent_uri,
                    RICO.agentRelationConnects,
                    RICO.agentIsConnectedToAgentRelation,
                    RICO.agentRelationConnects,
                    RICO.agentIsConnectedToAgentRelation,
                    RICO.isAgentAssociatedWithAgent,
                    RICO.isAgentAssociatedWithAgent,
                    desc,
                    start,
                    end,
                )

        predecessors = self._cw.execute(
            "Any AGENT, DESC, START WHERE "
            "R is ChronologicalRelation, R chronological_successor X,"
            "R chronological_predecessor AGENT, R description DESC,"
            "AGENT is AuthorityRecord,"
            "X is AuthorityRecord, X eid %(eid)s,"
            "R date_relation D?, D is DateEntity,"
            "D start_date START?",
            {"eid": self.entity.eid},
        )
        if predecessors:
            for agent_eid, desc, start in predecessors:
                predecessor_authority_record = self._cw.entity_from_eid(agent_eid)
                relation_uri = directional_relation_uri(
                    "temporal", predecessor_authority_record, self.entity
                )

                predecessor_uri = predecessor_authority_record.cw_adapt_to(
                    self.__regid__
                ).agent_uri

                yield from self.successor_triples(
                    relation_uri, predecessor_uri, self.agent_uri, desc, start
                )

        successors = self._cw.execute(
            "Any AGENT, DESC, START WHERE "
            "R is ChronologicalRelation, R chronological_predecessor X,"
            "R chronological_successor AGENT, R description DESC,"
            "AGENT is AuthorityRecord,"
            "X is AuthorityRecord, X eid %(eid)s,"
            "R date_relation D?, D is DateEntity,"
            "D start_date START?",
            {"eid": self.entity.eid},
        )
        if successors:
            for agent_eid, desc, start in successors:
                successor_authority_record = self._cw.entity_from_eid(agent_eid)
                relation_uri = directional_relation_uri(
                    "temporal", self.entity, successor_authority_record
                )

                successor_uri = successor_authority_record.cw_adapt_to(
                    self.__regid__
                ).agent_uri

                yield from self.successor_triples(
                    relation_uri, self.agent_uri, successor_uri, desc, start
                )

        yield (self.agent_uri, RICO.isOrWasDescribedBy, self.uri)
        function_rset = self._cw.execute(
            "Any F, NAME, START, END WHERE "
            "F is AgentFunction, F function_agent X,"
            "X is AuthorityRecord, X eid %(eid)s,"
            "F name NAME,"
            "F date_relation D?, D is DateEntity,"
            "D start_date START?, D end_date END?",
            {"eid": self.entity.eid},
        )

        if function_rset:
            for func_eid, func_name, start, end in function_rset:
                perf_uri = BNode()
                activity_uri = BNode()
                activity_type_uri = (
                    self._cw.entity_from_eid(func_eid).cw_adapt_to("rdf").uri
                )

                yield from self.relation_triples(
                    RICO.PerformanceRelation,
                    perf_uri,
                    activity_uri,
                    self.agent_uri,
                    RICO.performanceRelationHasSource,
                    RICO.activityIsSourceOfPerformanceRelation,
                    RICO.performanceRelationHasTarget,
                    RICO.agentIsTargetOfPerformanceRelation,
                    RICO.isOrWasPerformedBy,
                    RICO.performsOrPerformed,
                    None,
                    start,
                    end,
                )

                yield (activity_uri, RICO.hasActivityType, activity_type_uri)
                yield (activity_uri, RDF.type, RICO.Activity)
                yield (activity_type_uri, RDF.type, RICO.ActivityType)
                yield (activity_type_uri, RDFS.label, Literal(func_name))

        occupation_rset = self._cw.execute(
            "Any O, NAME, START, END WHERE "
            "O is Occupation, O occupation_agent X,"
            "X is AuthorityRecord, X eid %(eid)s,"
            "O term NAME,"
            "O date_relation D?, D is DateEntity,"
            "D start_date START?, D end_date END?",
            {"eid": self.entity.eid},
        )

        if occupation_rset:
            for occ_eid, occ_name, start, end in occupation_rset:
                perf_uri = BNode()
                occupation_uri = BNode()
                occupation_type_uri = (
                    self._cw.entity_from_eid(occ_eid).cw_adapt_to("rdf").uri
                )

                yield from self.relation_triples(
                    RICO.PerformanceRelation,
                    perf_uri,
                    occupation_uri,
                    self.agent_uri,
                    RICO.performanceRelationHasSource,
                    RICO.activityIsSourceOfPerformanceRelation,
                    RICO.performanceRelationHasTarget,
                    RICO.agentIsTargetOfPerformanceRelation,
                    RICO.isOrWasPerformedBy,
                    RICO.performsOrPerformed,
                    None,
                    start,
                    end,
                )
                yield (occupation_uri, RDF.type, RICO.Activity)
                yield (occupation_uri, RICO.hasActivityType, occupation_type_uri)
                yield (occupation_type_uri, RDF.type, RICO.OccupationType)
                yield (occupation_type_uri, RDFS.label, Literal(occ_name))

                yield (
                    self.agent_uri,
                    RICO.hasOrHadOccupationOfType,
                    occupation_type_uri,
                )
                yield (
                    occupation_type_uri,
                    RICO.isOrWasOccupationTypeOf,
                    self.agent_uri,
                )

        legal_status_rset = self._cw.execute(
            "Any S, NAME, START, END WHERE "
            "S is LegalStatus, S legal_status_agent X,"
            "X is AuthorityRecord, X eid %(eid)s,"
            "S term NAME,"
            "S date_relation D?, D is DateEntity,"
            "D start_date START?, D end_date END?",
            {"eid": self.entity.eid},
        )

        if legal_status_rset:
            for stat_eid, stat_name, start, end in legal_status_rset:
                relation_uri = BNode()
                legal_status_uri = (
                    self._cw.entity_from_eid(stat_eid).cw_adapt_to("rdf").uri
                )

                yield from self.relation_triples(
                    RICO.TypeRelation,
                    relation_uri,
                    legal_status_uri,
                    self.agent_uri,
                    RICO.typeRelationHasSource,
                    RICO.typeIsSourceOfTypeRelation,
                    RICO.typeRelationHasTarget,
                    RICO.thingIsTargetOfTypeRelation,
                    RICO.isOrWasLegalStatusOf,
                    RICO.hasOrHadLegalStatus,
                    None,
                    start,
                    end,
                )
                yield (legal_status_uri, RDF.type, RICO.LegalStatus)
                yield (legal_status_uri, RDFS.label, Literal(stat_name))

        mandate_rset = self._cw.execute(
            "Any M, NAME, DESC, START, END, NOTE, URL WHERE "
            "M is Mandate, M mandate_agent X,"
            "X is AuthorityRecord, X eid %(eid)s,"
            "M term NAME, M description DESC,"
            "M date_relation D?, D is DateEntity,"
            "D start_date START?, D end_date END?,"
            "M has_citation C?, C note NOTE?, C uri URL?",
            {"eid": self.entity.eid},
        )

        if mandate_rset:
            for (
                mand_eid,
                mand_name,
                mand_desc,
                start,
                end,
                note,
                mand_url,
            ) in mandate_rset:
                relation_uri = BNode()
                mandate_uri = self._cw.entity_from_eid(mand_eid).cw_adapt_to("rdf").uri

                yield from self.relation_triples(
                    RICO.MandateRelation,
                    relation_uri,
                    mandate_uri,
                    self.agent_uri,
                    RICO.mandateRelationHasSource,
                    RICO.mandateIsSourceOfMandateRelation,
                    RICO.mandateRelationHasTarget,
                    RICO.agentIsTargetOfMandateRelation,
                    RICO.authorizes,
                    RICO.authorizedBy,
                    mand_desc,
                    start,
                    end,
                )
                yield (mandate_uri, RDF.type, RICO.Mandate)
                if mand_name or note:
                    yield (
                        mandate_uri,
                        RDFS.label,
                        Literal(
                            f"{' -- '.join([x for x in (mand_name, note) if x is not None])}"
                        ),
                    )
                if mand_url:
                    yield (mandate_uri, RDFS.seeAlso, Literal(mand_url))

        place_entries = self._cw.execute(
            "Any P, PE, NAME, ROLE, LAT, LONG, START, END WHERE "
            "P is AgentPlace, P place_agent X,"
            "P role ROLE,"
            "X is AuthorityRecord, X eid %(eid)s,"
            "P place_entry_relation PE, PE is PlaceEntry,"
            "PE latitude LAT, PE longitude LONG,"
            "PE name NAME,"
            "P date_relation D?, D is DateEntity,"
            "D start_date START?, D end_date END?",
            {"eid": self.entity.eid},
        )

        if place_entries:
            for (
                rel_eid,
                place_eid,
                name,
                role,
                latitude,
                longitude,
                start,
                end,
            ) in place_entries:
                relation_uri = self._cw.entity_from_eid(rel_eid).cw_adapt_to("rdf").uri

                place_uri = self._cw.entity_from_eid(place_eid).cw_adapt_to("rdf").uri

                yield from self.relation_triples(
                    RICO.PlaceRelation,
                    relation_uri,
                    place_uri,
                    self.agent_uri,
                    RICO.placeRelationHasSource,
                    RICO.placeIsSourceOfPlaceRelation,
                    RICO.placeRelationHasTarget,
                    RICO.thingIsTargetOfPlaceRelation,
                    RICO.isOrWasLocationOf,
                    RICO.hasOrHadLocation,
                    None,
                    start,
                    end,
                )
                yield (relation_uri, RICO.type, Literal(role))
                yield (place_uri, RDF.type, RICO.Place)
                yield (place_uri, RICO.location, Literal(name))

                if latitude and longitude:
                    yield (
                        place_uri,
                        RICO.geographicalCoordinates,
                        Literal(f"{latitude} {longitude}"),
                    )

        name_entries = self._cw.execute(
            "Any N, NAME, TYPE, START, END WHERE "
            "N is NameEntry, N name_entry_for X,"
            "X is AuthorityRecord, X eid %(eid)s,"
            "N parts NAME, N form_variant TYPE,"
            "N date_relation D?, D is DateEntity,"
            "D start_date START?, D end_date END?",
            {"eid": self.entity.eid},
        )

        if name_entries:
            for (name_eid, name, variant, start, end) in name_entries:
                relation_uri = BNode()
                name_uri = self._cw.entity_from_eid(name_eid).cw_adapt_to("rdf").uri

                yield (self.agent_uri, RICO.name, Literal(name))
                yield (self.agent_uri, RICO.hasOrHadAgentName, name_uri)
                yield (name_uri, RICO.isOrWasAgentNameOf, self.agent_uri)
                yield (name_uri, RDF.type, RICO.AgentName)
                yield (name_uri, RICO.textualValue, Literal(name))
                if variant:
                    yield (name_uri, RICO.type, Literal(variant))
                if start:
                    yield (name_uri, RICO.usedFromDate, Literal(start))
                if end:
                    yield (name_uri, RICO.usedToDate, Literal(end))

        same_as = self._cw.execute(
            "Any URI WHERE "
            "R is IdentityRelation, R identity_from X,"
            "X is AuthorityRecord, X eid %(eid)s,"
            "R identity_to E, E is ExternalUri,"
            "E cwuri URI",
            {"eid": self.entity.eid},
        )

        if same_as:
            for uris in same_as:
                same_as_uri = uris[0]
                if _is_valid_uri(same_as_uri):
                    yield (self.agent_uri, OWL.sameAs, URIRef(same_as_uri))

    def record_triples(self):
        RICO = self._use_namespace(
            "rico", base_url="https://www.ica.org/standards/RiC/ontology#"
        )
        RDF = self._use_namespace("rdf")
        RICO_FORM = self._use_namespace(
            "ricoform",
            base_url="https://www.ica.org/standards/RiC/vocabularies/documentaryFormTypes#",
        )
        yield (self.inst_uri, RDF.type, RICO.Instantiation)
        yield (self.inst_uri, RICO.isInstantiationOf, self.uri)
        yield (self.inst_uri, RICO.identifier, Literal(self.entity.record_id))
        yield (self.uri, RICO.hasInstantiation, self.inst_uri)
        yield (self.agent_uri, RICO.isOrWasDescribedBy, self.uri)
        yield (self.uri, RICO.describesOrDescribed, self.agent_uri)
        yield (self.uri, RDF.type, RICO.Record)
        yield (
            self.uri,
            RICO.hasDocumentaryForm,
            RICO_FORM.AuthorityRecord,
        )

        rset = self._cw.execute(
            "Any A, TYPE, START, END, AGENTTYPE, AGENT, DESCR WHERE"
            "A is Activity, A generated X, X is AuthorityRecord, X eid %(eid)s,"
            "A type TYPE, A start START, A end END, A agent_type AGENTTYPE,"
            "A agent AGENT, A description DESCR",
            {"eid": self.entity.eid},
        )
        if rset:
            for act_eid, act_type, start, end, agent_type, agent, description in rset:
                activity_uri = self._cw.entity_from_eid(act_eid).cw_adapt_to("rdf").uri

                yield (self.uri, RICO.isOrWasAffectedBy, activity_uri)
                yield (activity_uri, RDF.type, RICO.Activity)
                yield (
                    activity_uri,
                    RICO.name,
                    Literal(f"{act_type} ({agent}, {agent_type})"),
                )
                if start:
                    yield (activity_uri, RICO.beginningDate, Literal(start))
                if start:
                    yield (activity_uri, RICO.endDate, Literal(end))
                if description:
                    yield (
                        activity_uri,
                        RICO.descriptiveNote,
                        Literal(remove_html_tags(description)),
                    )

        sources = self._cw.execute(
            "Any TITLE, URL WHERE"
            "S is EACSource, S source_agent X, X is AuthorityRecord, X eid %(eid)s,"
            "S title TITLE, S url URL",
            {"eid": self.entity.eid},
        )
        if sources:
            for title, url in sources:
                if url is not None and _is_valid_uri(url):
                    yield (self.uri, RICO.hasSource, URIRef(url))
                elif title:
                    yield (self.uri, RICO.source, Literal(title))

    def triples(self):
        yield from self.record_triples()
        yield from self.agent_triples()



# copyright 2015-2022 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
"""cubicweb-eac test for views."""

import datetime
import unittest

from rdflib import ConjunctiveGraph, Graph
from rdflib.compare import graph_diff

from cubicweb.devtools import BASE_URL
from cubicweb.devtools.testlib import CubicWebTC

from cubicweb_eac import testutils

from utils import add_entity_to_graph


class AuthorityRecordRdfTC(CubicWebTC):
    def setup_database(self):
        with self.admin_access.cnx() as cnx:
            copain = testutils.authority_record(cnx, "B123", "Toto")
            copine = testutils.authority_record(cnx, "B234", "Titi")
            person = testutils.authority_record(
                cnx,
                "A123",
                "Jean Jacques",
                kind="person",
                start_date=datetime.date(2010, 1, 1),
                end_date=datetime.date(2050, 5, 2),
                reverse_occupation_agent=cnx.create_entity(
                    "Occupation", term="fan de poules"
                ),
                reverse_history_agent=cnx.create_entity(
                    "History", text="<p>loutre gentille<p>", text_format="text/html"
                ),
                reverse_family_from=(
                    cnx.create_entity("FamilyRelation", family_to=copain, entry="Toto"),
                    cnx.create_entity("FamilyRelation", family_to=copine, entry="Titi"),
                ),
            )
            parent_organization = testutils.authority_record(
                cnx, "P123", "Toto Cie", kind="authority"
            )
            child_organization = testutils.authority_record(
                cnx, "P243", "Titi Cie", kind="authority"
            )
            organization = testutils.authority_record(
                cnx,
                "C123",
                "Entreprise",
                kind="authority",
                start_date=datetime.date(2010, 1, 1),
                end_date=datetime.date(2050, 5, 2),
            )
            cnx.create_entity(
                "HierarchicalRelation",
                hierarchical_child=(child_organization),
                hierarchical_parent=(organization),
                entry="Titi Cie",
            )
            cnx.create_entity(
                "HierarchicalRelation",
                hierarchical_child=(organization),
                hierarchical_parent=(parent_organization),
                entry="Titi Cie",
            )
            fam1 = testutils.authority_record(
                cnx, "F123", "Famille Pigeon", kind="family"
            )
            fam2 = testutils.authority_record(
                cnx, "F234", "Famille Poulet", kind="family"
            )
            cnx.create_entity(
                "FamilyRelation",
                family_from=fam1,
                family_to=fam2,
                entry="Famille Poulet",
            )
            cnx.create_entity(
                "FamilyRelation",
                family_from=fam1,
                family_to=person,
                entry="Jean Jacques",
            )
            cnx.create_entity(
                "FamilyRelation",
                family_from=person,
                family_to=fam1,
                entry="Famille Poulet",
            )
            cnx.commit()

    def compare_graphs(self, graph, eid_dict, target_ttl_file_name):
        with open(self.datapath(target_ttl_file_name), "r") as f:
            data = f.read()
            data = data.replace("{{BASE_URL}}", BASE_URL)
            for key, value in eid_dict.items():
                data = data.replace("{{%s}}" % key, str(value))

            target_graph = Graph().parse(data=data, format="ttl")
            print(graph.serialize(format="ttl"))
            common, tested_only, target_only = graph_diff(graph, target_graph)
            print(target_only.serialize(format="ttl"))
            print(tested_only.serialize(format="ttl"))
            self.assertEqual(len(tested_only), 0)
            self.assertEqual(len(target_only), 0)

    def test_person_authority_schemaorg(self):
        with self.admin_access.repo_cnx() as cnx:
            entity = cnx.find("AuthorityRecord", record_id="A123").one()
            graph = ConjunctiveGraph()
            add_entity_to_graph(graph, entity, "rdf.schemaorg")
            self.compare_graphs(graph, {"eid": entity.eid}, "person_rdf.ttl")

    def test_organization_authority_schemaorg(self):
        with self.admin_access.repo_cnx() as cnx:
            entity = cnx.find("AuthorityRecord", record_id="C123").one()
            graph = ConjunctiveGraph()
            add_entity_to_graph(graph, entity, "rdf.schemaorg")
            self.compare_graphs(graph, {"eid": entity.eid}, "organization_rdf.ttl")

    def test_family_rico(self):
        with self.admin_access.repo_cnx() as cnx:
            rset = cnx.execute(
                "Any X, N, A WHERE X is AuthorityRecord, X record_id %(id)s,"
                "R is FamilyRelation, R family_from X,"
                "R family_to A, A agent_kind K, K name 'person',"
                "N name_entry_for X",
                {"id": "F123"},
            )
            eid, name, agent_person = rset[0]
            record = cnx.entity_from_eid(eid)

            rset = cnx.execute(
                "Any A WHERE X is AuthorityRecord, X record_id %(id)s,"
                "R is FamilyRelation, R family_from X,"
                "R family_to A, A agent_kind K, K name 'family'",
                {"id": "F123"},
            )
            family_eid = rset[0][0]
            graph = ConjunctiveGraph()
            add_entity_to_graph(graph, record, "rdf.rico")
            self.compare_graphs(
                graph,
                {
                    "eid": record.eid,
                    "p_eid": agent_person,
                    "name_eid": name,
                    "ar_eid": family_eid,
                },
                "family_rico.ttl",
            )

    def test_person_rico(self):
        with self.admin_access.repo_cnx() as cnx:
            rset = cnx.execute(
                "Any X, N, O WHERE X is AuthorityRecord, X record_id %(id)s,"
                "O occupation_agent X,"
                "N name_entry_for X",
                {"id": "A123"},
            )
            eid, name, occupation = rset[0]
            record = cnx.entity_from_eid(eid)

            rset = cnx.execute(
                "Any R, A WHERE X is AuthorityRecord, X record_id %(id)s,"
                "R is FamilyRelation, R family_from X,"
                "R family_to A, A agent_kind K, K name 'family'",
                {"id": "A123"},
            )
            rel_family, family_eid = rset[0]

            rset = cnx.execute(
                "Any A, R WHERE X is AuthorityRecord, X record_id %(id)s,"
                "R is FamilyRelation, R family_from X,"
                "R family_to A, A agent_kind K, K name 'person'",
                {"id": "A123"},
            )
            fam1, famrel1 = rset[0]
            fam2, famrel2 = rset[1]

            graph = ConjunctiveGraph()
            add_entity_to_graph(graph, record, "rdf.rico")
            self.compare_graphs(
                graph,
                {
                    "eid": eid,
                    "occupation": occupation,
                    "name": name,
                    "fam1": fam1,
                    "fam2": fam2,
                    "famrel1": famrel1,
                    "famrel2": famrel2,
                    "familyrel": rel_family,
                    "family": family_eid,
                },
                "person_rico.ttl",
            )

    def test_organization_rico(self):
        with self.admin_access.repo_cnx() as cnx:
            rset = cnx.execute(
                "Any X, N WHERE X is AuthorityRecord, X record_id %(id)s,"
                "N name_entry_for X",
                {"id": "C123"},
            )
            eid, name = rset[0]
            record = cnx.entity_from_eid(eid)

            rset = cnx.execute(
                "Any A, R WHERE X is AuthorityRecord, X record_id %(id)s,"
                "R is HierarchicalRelation, R hierarchical_child X,"
                "R hierarchical_parent A",
                {"id": "C123"},
            )
            child_of, child_of_rel = rset[0]

            rset = cnx.execute(
                "Any A, R WHERE X is AuthorityRecord, X record_id %(id)s,"
                "R is HierarchicalRelation, R hierarchical_parent X,"
                "R hierarchical_child A",
                {"id": "C123"},
            )
            parent_of, parent_of_rel = rset[0]

            graph = ConjunctiveGraph()
            add_entity_to_graph(graph, record, "rdf.rico")
            self.compare_graphs(
                graph,
                {
                    "eid": eid,
                    "name": name,
                    "parent_of": parent_of,
                    "parent_of_rel": parent_of_rel,
                    "child_of": child_of,
                    "child_of_rel": child_of_rel,
                },
                "organization_rico.ttl",
            )


if __name__ == "__main__":
    unittest.main()



# coding: utf-8
# copyright 2016-2022 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
"""Tests for AuthorityRecord entities"""


from lxml import etree

from cubicweb.devtools.testlib import CubicWebTC

from cubicweb_eac import testutils


class EACExportTC(CubicWebTC):
    """Unit tests for EAC-CPF exports."""

    def test_richstring_plain(self):
        with self.admin_access.cnx() as cnx:
            alice = testutils.authority_record(cnx, "T-01", "Alice")
            desc = "ding\nlikes rabbits"
            mandate = cnx.create_entity(
                "Mandate",
                term="ding-girl",
                description=desc,
                description_format="text/plain",
                mandate_agent=alice,
            )
            cnx.commit()
            serializer = alice.cw_adapt_to("EAC-CPF")
            line1, line2 = serializer._eac_richstring_paragraph_elements(
                mandate, "description"
            )
        self.assertEqual(line1.tag, "p")
        self.assertEqual(line1.text, "ding")
        self.assertEqual(line2.tag, "p")
        self.assertEqual(line2.text, "likes rabbits")

    def test_richstring_html_simple(self):
        with self.admin_access.cnx() as cnx:
            alice = testutils.authority_record(cnx, "T-01", "Alice")
            desc = "<span>ding</span>"
            mandate = cnx.create_entity(
                "Mandate",
                term="ding-girl",
                description=desc,
                description_format="text/html",
                mandate_agent=alice,
            )
            cnx.commit()
            serializer = alice.cw_adapt_to("EAC-CPF")
            (tag,) = serializer._eac_richstring_paragraph_elements(
                mandate, "description"
            )
        self.assertEqual(tag.tag, "span")
        self.assertIn(desc, etree.tostring(tag, encoding="unicode"))

    def test_richstring_html_multiple_elements(self):
        with self.admin_access.cnx() as cnx:
            alice = testutils.authority_record(cnx, "T-01", "Alice")
            desc = ["<h1>she <i>rules!</i></h1>", '<a href="1">pif</a>']
            mandate = cnx.create_entity(
                "Mandate",
                term="chairgirl",
                description="".join(desc),
                description_format="text/html",
                mandate_agent=alice,
            )
            cnx.commit()
            serializer = alice.cw_adapt_to("EAC-CPF")
            h1, a = serializer._eac_richstring_paragraph_elements(
                mandate, "description"
            )
        self.assertEqual(h1.tag, "h1")
        self.assertEqual(a.tag, "a")
        self.assertIn(etree.tostring(h1, encoding="unicode"), desc[0])
        self.assertIn(etree.tostring(a, encoding="unicode"), desc[1])

    def test_richstring_markdown(self):
        with self.admin_access.cnx() as cnx:
            alice = testutils.authority_record(cnx, "T-01", "Alice")
            desc = "[pif](http://gadget.com) is *red*"
            desc_html = '<a href="http://gadget.com">pif</a> ' "is <em>red</em>"
            mandate = cnx.create_entity(
                "Mandate",
                term="w",
                description=desc,
                description_format="text/markdown",
                mandate_agent=alice,
            )
            cnx.commit()
            serializer = alice.cw_adapt_to("EAC-CPF")
            (tag,) = serializer._eac_richstring_paragraph_elements(
                mandate, "description"
            )
        self.assertEqual(tag.tag, "p")
        self.assertIn(desc_html, etree.tostring(tag, encoding="unicode"))

    def test_richstring_rest(self):
        with self.admin_access.cnx() as cnx:
            alice = testutils.authority_record(cnx, "T-01", "Alice")
            desc = "`pif <http://gadget.com>`_ is *red*"
            desc_html = (
                '<a class="reference" href="http://gadget.com">pif</a> '
                "is <em>red</em>"
            )
            mandate = cnx.create_entity(
                "Mandate",
                term="w",
                description=desc,
                description_format="text/rest",
                mandate_agent=alice,
            )
            cnx.commit()
            serializer = alice.cw_adapt_to("EAC-CPF")
            (ptag,) = serializer._eac_richstring_paragraph_elements(
                mandate, "description"
            )
        self.assertEqual(ptag.tag, "p")
        self.assertIn(desc_html, etree.tostring(ptag, encoding="unicode"))

    def test_richstring_empty(self):
        def check(authority_record):
            serializer = authority_record.cw_adapt_to("EAC-CPF")
            res = serializer._eac_richstring_paragraph_elements(mandate, "description")
            self.assertEqual(res, [])

        with self.admin_access.cnx() as cnx:
            alice = testutils.authority_record(cnx, "T-01", "Alice")
            mandate = cnx.create_entity(
                "Mandate", term="w", description=None, mandate_agent=alice
            )
            cnx.commit()
            check(alice)
        with self.admin_access.cnx() as cnx:
            cnx.execute('SET X description_format "text/rest" WHERE X is Mandate')
            cnx.commit()
            authority_record = cnx.execute(
                'Any X WHERE N name_entry_for X, N parts "Alice"'
            ).one()
            check(authority_record)

    def test_richstring_non_none_non_html(self):
        with self.admin_access.cnx() as cnx:
            arecord = testutils.authority_record(cnx, "R-01", "R")
            mandate = cnx.create_entity(
                "Mandate",
                term="w",
                description=" ",
                description_format="text/markdown",
                mandate_agent=arecord,
            )
            cnx.commit()
            serializer = arecord.cw_adapt_to("EAC-CPF")
            element = serializer.mandate_element(mandate)
            self.assertEqual(["term"], [child.tag for child in element])


if __name__ == "__main__":
    import unittest

    unittest.main()



from cubicweb import ValidationError
from cubicweb.devtools.testlib import CubicWebTC

from cubicweb_eac import testutils


class TernaryRelationDeletionHookTC(CubicWebTC):
    def setup_database(self):
        with self.admin_access.cnx() as cnx:
            self.arecord1_eid = testutils.authority_record(cnx, "T-01", "alice").eid
            self.arecord2_eid = testutils.authority_record(cnx, "T-02", "bob").eid
            self.arecord3_eid = testutils.authority_record(cnx, "T-03", "charles").eid
            cnx.create_entity(
                "AssociationRelation",
                entry="alice and bob are friends",
                association_from=self.arecord1_eid,
                association_to=self.arecord2_eid,
            )
            cnx.create_entity(
                "AssociationRelation",
                entry="bob knows charles",
                association_from=self.arecord2_eid,
                association_to=self.arecord3_eid,
            )
            cnx.create_entity(
                "ChronologicalRelation",
                entry="alice is bob's mother",
                chronological_predecessor=self.arecord1_eid,
                chronological_successor=self.arecord2_eid,
            )
            cnx.commit()

    def test(self):
        with self.admin_access.cnx() as cnx:
            with self.assertLogs("cubicweb.hook", level="INFO") as cm:
                cnx.execute(
                    "DELETE AuthorityRecord X WHERE X eid %(x)s",
                    {"x": self.arecord2_eid},
                )
            expected_msgs = [
                'deleting "alice and bob are friends" as association_to-object is being deleted',
                'deleting "bob knows charles" as association_from-object is being deleted',
                'deleting "alice is bob\'s mother" as chronological_successor-object is being deleted',  # noqa: E501
            ]
            self.assertCountEqual([r.message for r in cm.records], expected_msgs)
            cnx.commit()
            rset = cnx.find("AssociationRelation")
            self.assertFalse(rset)

    def test_unauthorized(self):
        with self.admin_access.cnx() as cnx:
            with self.temporary_permissions(
                AssociationRelation={"delete": ()}, ChronologicalRelation={"delete": ()}
            ):
                with self.assertRaises(ValidationError) as cm:
                    cnx.execute(
                        "DELETE AuthorityRecord X WHERE X eid %(x)s",
                        {"x": self.arecord2_eid},
                    )
        errors = str(cm.exception).splitlines()[1:]  # first line contain entity's eid.
        expected_msgs = [
            '* chronological_successor: "alice is bob\'s mother" would need to be deleted alongside the AuthorityRecord but this is disallowed',  # noqa: E501
            '* association_to: "alice and bob are friends" would need to be deleted alongside the AuthorityRecord but this is disallowed',  # noqa: E501
            '* association_from: "bob knows charles" would need to be deleted alongside the AuthorityRecord but this is disallowed',  # noqa: E501
        ]
        self.assertCountEqual(errors, expected_msgs)


if __name__ == "__main__":
    import unittest

    unittest.main()



# copyright 2016-2022 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
"""cubicweb-ctl plugin introducing eac-import command."""

from os.path import basename
from time import time

from cubicweb.toolsutils import Command, underline_title
from cubicweb.cwctl import CWCTL
from cubicweb.utils import admincnx
from cubicweb.dataimport.importer import SimpleImportLog
from cubicweb.web.views.cwsources import REVERSE_SEVERITIES

from cubicweb_eac.sobjects import init_extid2eid_index


class ImportEacData(Command):
    """Import some EAC files.

    <instance>
      identifier of the instance into which the scheme will be imported. Should use the eac cube.

    <filepath>
      path to the EAC files to import.

    """

    arguments = "[options] <instance> <filepath>..."
    name = "eac-import"
    min_args = 2

    def run(self, args):
        print("\n%s" % underline_title("Importing EAC files"))
        appid = args[0]
        with admincnx(appid) as cnx:
            eac_import_files(cnx, args[1:])
        cnx.repo.shutdown()


CWCTL.register(ImportEacData)


def eac_import_files(cnx, fpaths, store=None, **kwargs):
    """High-level import function, given an connection to a cubicweb repository and a list of files
    to import.
    """
    start_time = time()
    imported = created = updated = 0
    if store is None:
        store = _store(cnx)
    service = cnx.vreg["services"].select("eac.import", cnx, **kwargs)
    extid2eid = init_extid2eid_index(cnx, cnx.repo.system_source)
    try:
        for fpath in fpaths:
            _created, _updated = eac_import_file(service, store, fpath, extid2eid)
            if _created or _updated:
                imported += 1
                created += len(_created)
                updated += len(_updated)
            store.flush()
            store.commit()
    finally:
        store.finish()

    output_str = (
        "\nImported {imported}/{total} files ({created} entities + "
        "{updated} updates) in {time:.1f} seconds using {store}"
    )
    print(
        output_str.format(
            imported=imported,
            created=created,
            updated=updated,
            total=len(fpaths),
            time=time() - start_time,
            store=store.__class__.__name__,
        )
    )


def eac_import_file(service, store, fpath, extid2eid, raise_on_error=False):
    import_log = MyImportLog(basename(fpath))
    with open(fpath) as stream:
        try:
            created, updated, record_eid, not_visited = service.import_eac_stream(
                stream, import_log, extid2eid=extid2eid, store=store
            )
            return created, updated
        except Exception:
            if raise_on_error:
                raise
            return 0, 0


class MyImportLog(SimpleImportLog):
    def _log(self, severity, msg, path, line):
        if isinstance(msg, str):
            msg = msg.encode("utf8")
        print(
            "[{severity}] {path}:{line}: {msg}".format(
                severity=REVERSE_SEVERITIES[severity],
                path=self.filename,
                line=line or 0,
                msg=msg,
            )
        )


def _store(cnx):
    if cnx.repo.system_source.dbdriver == "postgres":
        from cubicweb.dataimport.stores import MetadataGenerator
        from cubicweb.dataimport.massive_store import MassiveObjectStore

        MetadataGenerator.META_RELATIONS = MetadataGenerator.META_RELATIONS - set(
            ["owned_by", "created_by"]
        )
        metagen = MetadataGenerator(cnx)
        return MassiveObjectStore(cnx, metagen=metagen, eids_seq_range=1000)
    else:
        from cubicweb.dataimport.stores import NoHookRQLObjectStore

        return NoHookRQLObjectStore(cnx)



add_attribute("ChronologicalRelation", "start_date", commit=False)
add_attribute("ChronologicalRelation", "end_date")

for etype in ("AssociationRelation", "ChronologicalRelation", "HierarchicalRelation"):
    add_attribute(etype, "entry")

add_entity_type("NameEntry")
with cnx.deny_all_hooks_but("metadata"):
    for record in rql(
        "Any X,XN WHERE X is AuthorityRecord, X name XN", ask_confirm=False
    ).entities():
        create_entity(
            "NameEntry",
            parts=record.name,
            form_variant="authorized",
            name_entry_for=record,
            ask_confirm=False,
        )
commit(ask_confirm=False)
drop_attribute("AuthorityRecord", "name")

add_attribute("Activity", "agent")

sync_schema_props_perms("has_citation", syncperms=False)



import doctest

from cubicweb.devtools.testlib import CubicWebTC

from cubicweb_eac import testutils

FLAGS = doctest.REPORT_UDIFF


class EACExportFunctionalTests(CubicWebTC, testutils.XmlTestMixin):
    """Functional tests for EAC-CPF export."""

    def setUp(self):
        super(EACExportFunctionalTests, self).setUp()
        self.globs = globals().copy()
        self.globs["self"] = self

    def _test(self, filename):
        with self.admin_access.cnx() as cnx:
            self.globs["cnx"] = cnx
            failure_count, test_count = doctest.testfile(
                filename, globs=self.globs, optionflags=FLAGS
            )
            if failure_count:
                self.fail(
                    "{} failures of {} in {} (check report)".format(
                        failure_count, test_count, filename
                    )
                )

    def test_simple(self):
        self._test("export-simple.rst")

    def test_roundtrip(self):
        self._test("export-roundtrip.rst")


if __name__ == "__main__":
    import unittest

    unittest.main()



add_entity_type("HistoricalEvent")
add_entity_type("Convention")
add_entity_type("ParallelNames")
add_entity_type("EACFunctionRelation")
add_entity_type("DateEntity")
add_entity_type("PlaceEntry")
add_entity_type("IdentityRelation")
add_entity_type("FamilyRelation")

# Attributes to update
add_attribute("EACResourceRelation", "xml_attributes")
add_attribute("EACResourceRelation", "relation_entry")
add_attribute("PostalAddress", "raw_address")

add_attribute("History", "abstract")

# migrate AgentPlace

for (agent_eid, name, exturi_eid,) in cnx.execute(
    "Any X, N, E WHERE X is AgentPlace, X name N, X equivalent_concept E?"
):
    cnx.create_entity(
        "PlaceEntry",
        place=name,
        reverse_place_entry_relation=agent_eid,
        equivalent_concept=exturi_eid,
    )

drop_attribute("AgentPlace", "name")
drop_relation_definition("AgentPlace", "equivalent_concept", ("ExternalUri", "Concept"))

cnx.commit()


for attrib in (
    "language",
    "preferred_form",
    "alternative_form",
    "authorized_form",
    "script_code",
):
    add_attribute("NameEntry", attrib)

add_relation_type("date_relation")

for etype in (
    "AssociationRelation",
    "ChronologicalRelation",
    "HierarchicalRelation",
    "Mandate",
    "LegalStatus",
    "HistoricalEvent",
    "Occupation",
    "EACFunctionRelation",
    "EACResourceRelation",
):
    subjrels = [s.type for s in cnx.vreg.schema.eschema(etype).subjrels]
    if "start_date" and "end_date" in subjrels:
        rql(
            """INSERT DateEntity D:
            E date_relation D, D start_date SD, D end_date ED WHERE
            E is {},
            E start_date SD, E end_date ED""".format(
                etype
            )
        )
        drop_attribute(etype, "start_date")
        drop_attribute(etype, "end_date")

add_relation_definition("NameEntry", "date_relation", "DateEntity")
add_relation_definition("Structure", "has_citation", "Citation")

for etype in (
    "GeneralContext",
    "Mandate",
    "Occupation",
    "History",
    "AgentFunction",
    "LegalStatus",
    "AgentPlace",
    "Structure",
):
    add_attribute(etype, "items")

add_relation_type("place_entry_relation")

add_attribute("Activity", "agent_type")

add_attribute("AuthorityRecord", "languages")

sync_schema_props_perms("record_id")

add_relation_definition(
    "EACResourceRelation", "resource_relation_resource", "AuthorityRecord"
)

add_relation_definition(
    "EACFunctionRelation", "function_relation_function", "AuthorityRecord"
)



# coding: utf-8
# copyright 2015-2022 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
"""cubicweb-eac unit tests for dataimport"""

import datetime
from io import BytesIO
from itertools import count
from os.path import join, dirname
import unittest

from lxml import etree

from cubicweb import NoResultError
from cubicweb.dataimport.importer import ExtEntity, SimpleImportLog
from cubicweb.devtools.testlib import CubicWebTC

from cubicweb_eac import dataimport, testutils

XML_TEST = """
<note test="test_value" test2="test2_value">
<to>Tove</to>
<from>Jani</from>
<heading>Reminder</heading>
<body>Hey!</body>
<empty></empty>
</note>
"""


def mock_(string):
    return string


def tolist(dic):
    """Transform sets in `dic` values as lists for easier comparison."""
    for k, v in dic.items():
        if isinstance(v, set):
            v = list(v)
        dic[k] = v
    return dic


def extentities2dict(entities):
    edict = {}
    for extentity in entities:
        edict.setdefault(extentity.etype, {})[extentity.extid] = extentity.values
    return edict


class EACXMLParserTC(unittest.TestCase):
    @classmethod
    def datapath(cls, *fname):
        """joins the object's datadir and `fname`"""
        return join(dirname(__file__), "data", *fname)

    def file_extentities(self, fname):
        fpath = self.datapath(fname)
        import_log = SimpleImportLog(fpath)
        # Use a predictable extid_generator.
        extid_generator = map(str, count()).__next__
        importer = dataimport.EACCPFImporter(
            fpath, import_log, mock_, extid_generator=extid_generator
        )
        return importer.external_entities()

    def test_parse_FRAD033_EAC_00001(self):
        _gen_extid = map(str, (x for x in count() if x not in (2, 38))).__next__
        expected = [
            (
                "EACOtherRecordId",
                _gen_extid(),
                {
                    "eac_other_record_id_of": set(
                        ["authorityrecord-FRAD033_EAC_00001"]
                    ),
                    "value": set(["1234"]),
                },
            ),
            (
                "EACOtherRecordId",
                _gen_extid(),
                {
                    "eac_other_record_id_of": set(
                        ["authorityrecord-FRAD033_EAC_00001"]
                    ),
                    "value": set(["ABCD"]),
                    "local_type": set(["letters"]),
                },
            ),
            (
                "EACSource",
                _gen_extid(),
                {
                    "source_agent": set(["authorityrecord-FRAD033_EAC_00001"]),
                    "title": set(["1. Ouvrages imprimés..."]),
                    "description": set(["des bouquins"]),
                    "description_format": set(["text/plain"]),
                },
            ),
            (
                "EACSource",
                _gen_extid(),
                {
                    "source_agent": set(["authorityrecord-FRAD033_EAC_00001"]),
                    "url": set(["http://archives.gironde.fr"]),
                    "title": set(["Site des Archives départementales de la Gironde"]),
                },
            ),
            (
                "Activity",
                _gen_extid(),
                {
                    "type": set(["create"]),
                    "agent_type": ["human"],
                    "generated": set(["authorityrecord-FRAD033_EAC_00001"]),
                    "start": set([datetime.datetime(2013, 4, 24, 5, 34, 41)]),
                    "end": set([datetime.datetime(2013, 4, 24, 5, 34, 41)]),
                    "description": set(["bla bla"]),
                    "description_format": set(["text/plain"]),
                },
            ),
            (
                "Activity",
                _gen_extid(),
                {
                    "generated": set(["authorityrecord-FRAD033_EAC_00001"]),
                    "type": set(["modify"]),
                    "agent_type": ["human"],
                    "start": set([datetime.datetime(2015, 1, 15, 7, 16, 33)]),
                    "end": set([datetime.datetime(2015, 1, 15, 7, 16, 33)]),
                    "agent": set(["Delphine Jamet"]),
                },
            ),
            (
                "Convention",
                _gen_extid(),
                {
                    "convention_of": ["authorityrecord-FRAD033_EAC_00001"],
                    "abbrev": set(["ISAAR(CPF)"]),
                    "has_citation": ["8"],
                    "description_format": set(["text/html"]),
                    "description": set(
                        [
                            '<p xmlns="urn:isbn:1-931666-33-4" '
                            'xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" '
                            'xmlns:xlink="http://www.w3.org/1999/xlink">Norme '
                            "ISAAR(CPF) du Conseil international des archives, "
                            "2e \xe9dition, 1996.</p>"
                        ]
                    ),
                },
            ),
            (
                "Citation",
                _gen_extid(),
                {
                    "uri": set(["http://www.ica.org"]),
                },
            ),
            (
                "Convention",
                _gen_extid(),
                {
                    "convention_of": ["authorityrecord-FRAD033_EAC_00001"],
                    "description_format": set(["text/html"]),
                    "description": set(
                        [
                            '<p xmlns="urn:isbn:1-931666-33-4" '
                            'xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" '
                            'xmlns:xlink="http://www.w3.org/1999/xlink">AFNOR '
                            "NF Z 44-060, octobre 1983, Catalogue "
                            "d\u2019auteurs et d\u2019anonymes : forme et\n          "
                            "structure des vedettes des collectivit\xe9s auteurs.</p>"
                        ]
                    ),
                },
            ),
            (
                "Convention",
                _gen_extid(),
                {
                    "convention_of": ["authorityrecord-FRAD033_EAC_00001"],
                    "description_format": set(["text/html"]),
                    "description": set(
                        [
                            '<p xmlns="urn:isbn:1-931666-33-4" '
                            'xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" '
                            'xmlns:xlink="http://www.w3.org/1999/xlink">Norme ISO 8601 '
                            ":2004 \xc9l\xe9ments de donn\xe9es et formats "
                            "d\u2019\xe9change -- \xc9change\n          "
                            "d\u2019information -- Repr\xe9sentation de la date et "
                            "de l\u2019heure.</p>"
                        ]
                    ),
                },
            ),
            (
                "AgentKind",
                "agentkind/authority",
                {"name": set(["authority"])},
            ),
            (
                "NameEntry",
                _gen_extid(),
                {
                    "parts": set(["Gironde, Conseil général"]),
                    "form_variant": set(["authorized"]),
                    "name_entry_for": set(["authorityrecord-FRAD033_EAC_00001"]),
                },
            ),
            (
                "NameEntry",
                _gen_extid(),
                {
                    "parts": set(["CG33"]),
                    "form_variant": set(["alternative"]),
                    "name_entry_for": set(["authorityrecord-FRAD033_EAC_00001"]),
                },
            ),
            (
                "ParallelNames",
                _gen_extid(),
                {
                    "parallel_names_of": set(["authorityrecord-FRAD033_EAC_00001"]),
                    "simple_name_relation": set(["15", "14"]),
                    "authorized_form": set(["AFNOR_Z44-060\n\t"]),
                },
            ),
            (
                "NameEntry",
                _gen_extid(),
                {
                    "script_code": set(["Latn"]),
                    "preferred_form": ["AFNOR_Z44-060\n\t  "],
                    "parts": set(
                        ["Institut international des droits de\n\t  l'homme\n\t  "]
                    ),
                    "language": set(["fr"]),
                },
            ),
            (
                "NameEntry",
                _gen_extid(),
                {
                    "script_code": set(["Latn"]),
                    "parts": set(
                        ["International institute of human\n\t  rights\n\t  "]
                    ),
                    "language": set(["en"]),
                },
            ),
            (
                "ParallelNames",
                _gen_extid(),
                {
                    "parallel_names_of": set(["authorityrecord-FRAD033_EAC_00001"]),
                    "date_relation": set(["17", "18", "19"]),
                    "simple_name_relation": set(["20", "21", "22"]),
                },
            ),
            (
                "DateEntity",
                _gen_extid(),
                {"start_date": set([datetime.date(1949, 1, 1)])},
            ),
            (
                "DateEntity",
                _gen_extid(),
                {"start_date": set([datetime.date(1950, 1, 1)])},
            ),
            (
                "DateEntity",
                _gen_extid(),
                {"start_date": set([datetime.date(1950, 1, 1)])},
            ),
            (
                "NameEntry",
                _gen_extid(),
                {"parts": ["Federal Chancellery\n\t  of Germany\n\t  "]},
            ),
            (
                "NameEntry",
                _gen_extid(),
                {"parts": ["Chancellerie f\xe9d\xe9rale\n\t  d'Allemagne\n\t  "]},
            ),
            (
                "NameEntry",
                _gen_extid(),
                {"parts": set(["BK\n\t  "])},
            ),
            (
                "PostalAddress",
                _gen_extid(),
                {
                    "street": set(["1 Esplanade Charles de Gaulle"]),
                    "postalcode": set(["33074"]),
                    "raw_address": set(
                        ["1 Esplanade Charles de Gaulle\n33074\n Bordeaux Cedex"]
                    ),
                    "city": set([" Bordeaux Cedex"]),
                },
            ),
            (
                "AgentPlace",
                _gen_extid(),
                {
                    "role": set(["siege"]),
                    "place_agent": set(["authorityrecord-FRAD033_EAC_00001"]),
                    "place_entry_relation": set(["25"]),
                    "place_address": set(["23"]),
                },
            ),
            (
                "PlaceEntry",
                _gen_extid(),
                {
                    "name": set(["Bordeaux (Gironde, France)"]),
                    "equivalent_concept": set(
                        ["http://catalogue.bnf.fr/ark:/12148/cb152418385"]
                    ),
                },
            ),
            (
                "AgentPlace",
                _gen_extid(),
                {
                    "place_agent": set(["authorityrecord-FRAD033_EAC_00001"]),
                    "place_entry_relation": set(["27"]),
                    "role": set(["domicile"]),
                },
            ),
            (
                "PlaceEntry",
                _gen_extid(),
                {
                    "latitude": set(["43.60426"]),
                    "local_type": set(["other"]),
                    "longitude": set(["1.44367"]),
                    "name": set(["Toulouse (France)"]),
                },
            ),
            (
                "AgentPlace",
                _gen_extid(),
                {
                    "place_agent": set(["authorityrecord-FRAD033_EAC_00001"]),
                    "role": set(["dodo"]),
                    "place_entry_relation": set(["29"]),
                },
            ),
            (
                "PlaceEntry",
                _gen_extid(),
                {
                    "name": set(["Lit"]),
                },
            ),
            (
                "LegalStatus",
                _gen_extid(),
                {
                    "term": set(["Collectivité territoriale"]),
                    "date_relation": set(["31"]),
                    "description": set(["Description du statut"]),
                    "description_format": set(["text/plain"]),
                    "legal_status_agent": set(["authorityrecord-FRAD033_EAC_00001"]),
                },
            ),
            (
                "DateEntity",
                _gen_extid(),
                {
                    "start_date": set([datetime.date(1234, 1, 1)]),
                    "end_date": set([datetime.date(3000, 1, 1)]),
                },
            ),
            (
                "Mandate",
                _gen_extid(),
                {
                    "term": set(["1. Constitutions françaises"]),
                    "description": set(["Description du mandat"]),
                    "description_format": set(["text/plain"]),
                    "mandate_agent": set(["authorityrecord-FRAD033_EAC_00001"]),
                },
            ),
            (
                "History",
                _gen_extid(),
                {
                    "abstract": set(["Test of an abstract element"]),
                    "has_citation": set(["39", "40"]),
                    "has_event": set(["36", "34"]),
                    "text": set(
                        [
                            "\n".join(
                                (
                                    '<p xmlns="urn:isbn:1-931666-33-4" '
                                    'xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" '
                                    'xmlns:xlink="http://www.w3.org/1999/xlink">{0}</p>'
                                ).format(text)
                                for text in [
                                    "La loi du 22 décembre 1789, en divisant ...",
                                    "L'inspecteur Canardo",
                                ]
                            )
                        ]
                    ),
                    "text_format": set(["text/html"]),
                    "items": set(
                        [
                            '<ul  >\n\t    <li>\n\t      <span style="font-'
                            '       style:italic">1450-1950\n\t      </span>\n\t'
                            "      (1929)\n\t    </li>\n\t    <li>\n\t      "
                            '<span style="font-style:italic">Globe\n\t      '
                            "Gliding\n\t      </span>\n\t      (1930)\n\t    </l"
                            'i>\n\t    <li>\n\t      <span style="font-     '
                            '  style:italic">Gems\n\t      </span>\n\t      '
                            "(1931)\n\t    </li>\n\t  </ul>\n      "
                        ]
                    ),
                    "items_format": set(["text/html"]),
                    "history_agent": set(["authorityrecord-FRAD033_EAC_00001"]),
                },
            ),
            (
                "HistoricalEvent",
                _gen_extid(),
                {
                    "date_relation": set(["35"]),
                    "event": [
                        "Left Mer and moved to the mainland.\n\t      "
                        "Worked at various jobs including canecutter\n\t      "
                        "and railway labourer.\n\t      "
                    ],
                },
            ),
            (
                "DateEntity",
                _gen_extid(),
                {
                    "end_date": [datetime.date(1957, 1, 1)],
                    "start_date": [datetime.date(1957, 1, 1)],
                },
            ),
            (
                "HistoricalEvent",
                _gen_extid(),
                {
                    "date_relation": set(["37"]),
                    "event": set(
                        [
                            "Union representative, Townsville-\n\t      "
                            "Mount Isa rail construction project.\n\t      "
                        ]
                    ),
                },
            ),
            (
                "DateEntity",
                _gen_extid(),
                {
                    "end_date": [datetime.date(1961, 1, 1)],
                    "start_date": [datetime.date(1960, 1, 1)],
                },
            ),
            (
                "Citation",
                _gen_extid(),
                {
                    "uri": set(
                        [
                            "http://www.assemblee-nationale.fr/histoire/images-decentralisation/"
                            "decentralisation/loi-du-22-decembre-1789-.pdf"
                        ]
                    )
                },
            ),
            (
                "Citation",
                _gen_extid(),
                {
                    "uri": set(["http://pifgadget"]),
                    "note": set(["Voir aussi pifgadget"]),
                },
            ),
            (
                "Structure",
                _gen_extid(),
                {
                    "description": set(
                        [
                            '<p xmlns="urn:isbn:1-931666-33-4" '
                            'xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" '
                            'xmlns:xlink="http://www.w3.org/1999/xlink">Pour accomplir '
                            "ses missions ...</p>"
                        ]
                    ),
                    "description_format": set(["text/html"]),
                    "has_citation": set(["42"]),
                    "structure_agent": set(["authorityrecord-FRAD033_EAC_00001"]),
                },
            ),
            (
                "Citation",
                _gen_extid(),
                {
                    "note": set(
                        [
                            "L'\xe9l\xe9ment Citation \xe0 fournir un lien vers un document "
                            "externe comme un\n               organigramme ou un arbre "
                            "g\xe9n\xe9alogique. Pour une pr\xe9sentation plus simple, "
                            "sous forme\n               de texte, on peut utiliser un "
                            "ou plusieurs \xe9l\xe9m."
                        ]
                    )
                },
            ),
            (
                "AgentFunction",
                _gen_extid(),
                {
                    "description": set(
                        [
                            '<p xmlns="urn:isbn:1-931666-33-4" '
                            'xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" '
                            'xmlns:xlink="http://www.w3.org/1999/xlink">Quatre grands '
                            "domaines de compétence...</p>"
                        ]
                    ),
                    "description_format": set(["text/html"]),
                    "function_agent": set(["authorityrecord-FRAD033_EAC_00001"]),
                },
            ),
            (
                "AgentFunction",
                _gen_extid(),
                {
                    "name": set(["action sociale"]),
                    "function_agent": set(["authorityrecord-FRAD033_EAC_00001"]),
                    "description": set(
                        [
                            '<p xmlns="urn:isbn:1-931666-33-4" '
                            'xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" '
                            'xmlns:xlink="http://www.w3.org/1999/xlink">1. Solidarité\n'
                            "            blablabla.</p>"
                        ]
                    ),
                    "description_format": set(["text/html"]),
                    "equivalent_concept": set(
                        ["http://data.culture.fr/thesaurus/page/ark:/67717/T1-200"]
                    ),
                },
            ),
            (
                "AgentFunction",
                _gen_extid(),
                {
                    "name": set(["environnement"]),
                    "function_agent": set(["authorityrecord-FRAD033_EAC_00001"]),
                    "equivalent_concept": set(
                        ["http://data.culture.fr/thesaurus/page/ark:/67717/T1-1074"]
                    ),
                },
            ),
            (
                "Occupation",
                _gen_extid(),
                {
                    "term": set(["Réunioniste"]),
                    "date_relation": set(["47"]),
                    "description": set(["Organisation des réunions ..."]),
                    "description_format": set(["text/plain"]),
                    "occupation_agent": set(["authorityrecord-FRAD033_EAC_00001"]),
                    "has_citation": set(["48"]),
                    "equivalent_concept": set(["http://pifgadget.com"]),
                },
            ),
            (
                "DateEntity",
                _gen_extid(),
                {
                    "start_date": set([datetime.date(1987, 1, 1)]),
                    "end_date": set([datetime.date(2099, 1, 1)]),
                },
            ),
            (
                "Citation",
                _gen_extid(),
                {
                    "note": set(["la bible"]),
                },
            ),
            (
                "GeneralContext",
                _gen_extid(),
                {
                    "content": set(
                        [
                            '<p xmlns="urn:isbn:1-931666-33-4" '
                            'xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" '
                            'xmlns:xlink="http://www.w3.org/1999/xlink">very famous</p>'
                        ]
                    ),
                    "content_format": set(["text/html"]),
                    "has_citation": set(["50"]),
                    "general_context_of": set(["authorityrecord-FRAD033_EAC_00001"]),
                },
            ),
            (
                "Citation",
                _gen_extid(),
                {
                    "note": set(["it's well known"]),
                },
            ),
            (
                "ExternalUri",
                "CG33-DIRADSJ",
                {
                    "uri": set(["CG33-DIRADSJ"]),
                    "cwuri": set(["CG33-DIRADSJ"]),
                },
            ),
            (
                "HierarchicalRelation",
                _gen_extid(),
                {
                    "entry": set(
                        [
                            "Gironde. Conseil général. Direction de l'administration et de "
                            "la sécurité juridique"
                        ]
                    ),
                    "date_relation": set(["52"]),
                    "description": set(["Coucou"]),
                    "description_format": set(["text/plain"]),
                    "hierarchical_parent": set(["CG33-DIRADSJ"]),
                    "hierarchical_child": set(["authorityrecord-FRAD033_EAC_00001"]),
                },
            ),
            (
                "DateEntity",
                _gen_extid(),
                {
                    "start_date": set([datetime.date(2008, 1, 1)]),
                    "end_date": set([datetime.date(2099, 1, 1)]),
                },
            ),
            (
                "ExternalUri",
                "whatever",
                {
                    "uri": set(["whatever"]),
                    "cwuri": set(["whatever"]),
                },
            ),
            (
                "ExternalUri",
                "/dev/null",
                {
                    "uri": set(["/dev/null"]),
                    "cwuri": set(["/dev/null"]),
                },
            ),
            (
                "ChronologicalRelation",
                _gen_extid(),
                {
                    "chronological_predecessor": set(["whatever"]),
                    "chronological_successor": set(
                        ["authorityrecord-FRAD033_EAC_00001"]
                    ),
                    "date_relation": set(["54"]),
                    "entry": set(["CG32"]),
                },
            ),
            (
                "DateEntity",
                _gen_extid(),
                {
                    "start_date": set([datetime.date(1917, 1, 1)]),
                    "end_date": set([datetime.date(2009, 1, 1)]),
                },
            ),
            (
                "ChronologicalRelation",
                _gen_extid(),
                {
                    "chronological_predecessor": set(
                        ["authorityrecord-FRAD033_EAC_00001"]
                    ),
                    "chronological_successor": set(["/dev/null"]),
                    "date_relation": set(["56"]),
                    "xml_wrap": set(
                        [
                            b'<gloups xmlns="urn:isbn:1-931666-33-4"'
                            b' xmlns:xsi="http://www.w3.org/2001/XML'
                            b'Schema-instance" xmlns:xlink="http://'
                            b'www.w3.org/1999/xlink">hips</gloups>'
                        ]
                    ),
                    "entry": set(["Trash"]),
                },
            ),
            (
                "DateEntity",
                _gen_extid(),
                {"start_date": set([datetime.date(2042, 1, 1)])},
            ),
            (
                "IdentityRelation",
                _gen_extid(),
                {
                    "date_relation": ["58"],
                    "entry": ["Trash"],
                    "identity_from": ["authorityrecord-FRAD033_EAC_00001"],
                    "identity_to": ["/dev/null"],
                    "xml_wrap": set(
                        [
                            b'<gloups xmlns="urn:isbn:1-931666-33-4" '
                            b'xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" '
                            b'xmlns:xlink="http://www.w3.org/1999/xlink">hips</gloups>'
                        ]
                    ),
                },
            ),
            ("DateEntity", _gen_extid(), {"start_date": [datetime.date(2042, 1, 1)]}),
            (
                "FamilyRelation",
                _gen_extid(),
                {
                    "date_relation": ["60"],
                    "entry": ["CG32"],
                    "family_from": ["authorityrecord-FRAD033_EAC_00001"],
                    "family_to": ["whatever"],
                },
            ),
            (
                "DateEntity",
                _gen_extid(),
                {
                    "end_date": [datetime.date(2009, 1, 1)],
                    "start_date": [datetime.date(1917, 1, 1)],
                },
            ),
            (
                "AssociationRelation",
                _gen_extid(),
                {
                    "association_from": set(["authorityrecord-FRAD033_EAC_00001"]),
                    "association_to": set(["agent-x"]),
                },
            ),
            (
                "EACResourceRelation",
                _gen_extid(),
                {
                    "agent_role": set(["creatorOf"]),
                    "date_relation": set(["63"]),
                    "xml_attributes": set(
                        [
                            '{"{http://www.w3.org/1999/xlink}actuate": "onRequest", '
                            '"{http://www.w3.org/1999/xlink}show": "new", '
                            '"{http://www.w3.org/1999/xlink}type": "simple"}'
                        ]
                    ),
                    "relation_entry": set(
                        [
                            "Gironde. Conseil g\xe9n\xe9ral. Direction de"
                            " l'administration et de la s\xe9curit\xe9 juridique"
                        ]
                    ),
                    "resource_role": set(["Fonds d'archives"]),
                    "resource_relation_resource": set(
                        ["http://gael.gironde.fr/ead.html?id=FRAD033_IR_N"]
                    ),
                    "resource_relation_agent": set(
                        ["authorityrecord-FRAD033_EAC_00001"]
                    ),
                    "xml_wrap": set(
                        [
                            b'<he xmlns="urn:isbn:1-931666-33-4" '
                            b'xmlns:xlink="http://www.w3.org/1999'
                            b'/xlink" xmlns:xsi="http://www.w3.org'
                            b'/2001/XMLSchema-instance">joe</he>'
                        ]
                    ),
                },
            ),
            (
                "DateEntity",
                _gen_extid(),
                {
                    "start_date": set([datetime.date(1673, 1, 1)]),
                    "end_date": set([datetime.date(1963, 1, 1)]),
                },
            ),
            (
                "EACFunctionRelation",
                _gen_extid(),
                {
                    "description": set(
                        [
                            '<p xmlns="urn:isbn:1-931666-33-4" xmlns:xsi="http:/'
                            '/www.w3.org/2001/XMLSchema-instance" xmlns:xlink="http:'
                            '//www.w3.org/1999/xlink">The management of the University'
                            "'s\n\t  communication with its alumni.\n\t  </p>"
                        ]
                    ),
                    "r_type": set(["performs"]),
                    "description_format": set(["text/html"]),
                    "function_relation_agent": set(
                        ["authorityrecord-FRAD033_EAC_00001"]
                    ),
                    "function_relation_function": set(
                        ["http://gael.gironde.fr/ead.html?" "id=FRAD033_IR_N"]
                    ),
                    "relation_entry": set(
                        [
                            "Alumni communication\n\tmanagement, "
                            "University of\n\tGlasgow\n\t"
                        ]
                    ),
                    "xml_wrap": set(
                        [
                            b'<mods xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"'
                            b' xmlns="urn:isbn:1-931666-33-4" xmlns:xlink="http://www.w3'
                            b'.org/1999/xlink" xsi:schemaLocation="http://www.loc.gov'
                            b"/mods/v3 http:         //www.loc.gov/mods/v3/mods-3-3.xsd"
                            b'">\n\t    <titleInfo>\n\t      <title>Artisti trentini'
                            b" tra le due\n\t      guerre\n\t      </title>\n\t    </titleInfo"
                            b'>\n\t    <name>\n\t      <namePart type="given">Nicoletta'
                            b'\n\t      </namePart>\n\t      <namePart type="family'
                            b'">Boschiero\n\t      </namePart>\n\t      <role>\n\t\t<roleTerm'
                            b' type="text">autore\n\t\t</roleTerm>\n\t      </role>\n\t'
                            b"    </name>\n\t  </mods>\n\t"
                        ]
                    ),
                    "xml_attributes": set(
                        [
                            '{"{http://www.w3.org/1999/xlink}actuate": '
                            '"onLoad", "{http://www.w3.org/1999/xlink}arcrole": '
                            '"http://test_arcrole.lol.com", '
                            '"{http://www.w3.org/1999/xlink}role": '
                            '"http://test_role.lmao.com"}'
                        ]
                    ),
                },
            ),
            (
                "EACFunctionRelation",
                _gen_extid(),
                {
                    "function_relation_function": set(["FRAD033_IR_N"]),
                    "function_relation_agent": set(
                        ["authorityrecord-FRAD033_EAC_00001"]
                    ),
                    "description": set(
                        [
                            '<p xmlns="urn:isbn:1-931666-33-4" '
                            'xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" '
                            'xmlns:xlink="http://www.w3.org/1999/xlink">'
                            "The second responsibility of the\n\t  "
                            "Department is to control the establishment\n\t  "
                            "and abolishment of schools.\n\t  </p>"
                        ]
                    ),
                    "r_type": set(["controls"]),
                    "description_format": set(["text/html"]),
                    "date_relation": set(["66"]),
                    "relation_entry": set(
                        ["Establishment and abolishment\n\tof schools\n\t"]
                    ),
                    "xml_attributes": set(["{}"]),
                },
            ),
            (
                "DateEntity",
                _gen_extid(),
                {
                    "start_date": set([datetime.date(1922, 1, 1)]),
                    "end_date": set([datetime.date(2001, 1, 1)]),
                },
            ),
            (
                "EACFunctionRelation",
                _gen_extid(),
                {
                    "function_relation_agent": set(
                        ["authorityrecord-FRAD033_EAC_00001"]
                    ),
                    "description": set(
                        [
                            '<p xmlns="urn:isbn:1-931666-33-4" '
                            'xmlns:xsi="http://www.w3.org/2001/X'
                            'MLSchema-instance" xmlns:xlink="http://ww'
                            'w.w3.org/1999/xlink">Some description'
                            "\n            </p>"
                        ]
                    ),
                    "function_relation_function": set(["ONLY_XLINK"]),
                    "description_format": set(["text/html"]),
                    "relation_entry": set(["Some relation entry\n          "]),
                    "xml_attributes": set(["{}"]),
                    "date_relation": ["68"],
                },
            ),
            (
                "DateEntity",
                _gen_extid(),
                {
                    "start_date": set([datetime.date(1922, 1, 1)]),
                    "end_date": set([datetime.date(2001, 1, 1)]),
                },
            ),
            (
                "EACFunctionRelation",
                _gen_extid(),
                {
                    "function_relation_agent": set(
                        ["authorityrecord-FRAD033_EAC_00001"]
                    ),
                    "description": set(
                        [
                            '<p xmlns="urn:isbn:1-931666-33-4" '
                            'xmlns:xsi="http://www.w3.org/2001/X'
                            'MLSchema-instance" xmlns:xlink="http://ww'
                            'w.w3.org/1999/xlink">Some description'
                            "\n            </p>"
                        ]
                    ),
                    "r_type": set(["ONLY_RELATION_TYPE"]),
                    "description_format": set(["text/html"]),
                    "relation_entry": set(["Some relation entry\n          "]),
                    "xml_attributes": set(["{}"]),
                    "date_relation": ["70"],
                },
            ),
            (
                "DateEntity",
                _gen_extid(),
                {
                    "start_date": set([datetime.date(1922, 1, 1)]),
                    "end_date": set([datetime.date(2001, 1, 1)]),
                },
            ),
            (
                "ExternalUri",
                "ONLY_XLINK",
                {"uri": set(["ONLY_XLINK"]), "cwuri": set(["ONLY_XLINK"])},
            ),
            (
                "ExternalUri",
                "FRAD033_IR_N",
                {"uri": set(["FRAD033_IR_N"]), "cwuri": set(["FRAD033_IR_N"])},
            ),
            (
                "ExternalUri",
                "http://gael.gironde.fr/ead.html?id=FRAD033_IR_N",
                {
                    "uri": set(["http://gael.gironde.fr/ead.html?id=FRAD033_IR_N"]),
                    "cwuri": set(["http://gael.gironde.fr/ead.html?id=FRAD033_IR_N"]),
                },
            ),
            (
                "ExternalUri",
                "agent-x",
                {"uri": set(["agent-x"]), "cwuri": set(["agent-x"])},
            ),
            (
                "ExternalUri",
                "http://data.culture.fr/thesaurus/page/ark:/67717/T1-200",
                {
                    "uri": set(
                        ["http://data.culture.fr/thesaurus/page/ark:/67717/T1-200"]
                    ),
                    "cwuri": set(
                        ["http://data.culture.fr/thesaurus/page/ark:/67717/T1-200"]
                    ),
                },
            ),
            (
                "ExternalUri",
                "http://data.culture.fr/thesaurus/page/ark:/67717/T1-1074",
                {
                    "uri": set(
                        ["http://data.culture.fr/thesaurus/page/ark:/67717/T1-1074"]
                    ),
                    "cwuri": set(
                        ["http://data.culture.fr/thesaurus/page/ark:/67717/T1-1074"]
                    ),
                },
            ),
            (
                "ExternalUri",
                "http://catalogue.bnf.fr/ark:/12148/cb152418385",
                {
                    "uri": set(["http://catalogue.bnf.fr/ark:/12148/cb152418385"]),
                    "cwuri": set(["http://catalogue.bnf.fr/ark:/12148/cb152418385"]),
                },
            ),
            (
                "ExternalUri",
                "http://pifgadget.com",
                {
                    "uri": set(["http://pifgadget.com"]),
                    "cwuri": set(["http://pifgadget.com"]),
                },
            ),
            (
                "AuthorityRecord",
                "authorityrecord-FRAD033_EAC_00001",
                {
                    "isni": set(["22330001300016"]),
                    "languages": ["English, Spanish"],
                    "start_date": set([datetime.date(1800, 1, 1)]),
                    "end_date": set([datetime.date(2099, 1, 1)]),
                    "agent_kind": set(["agentkind/authority"]),
                    "record_id": set(["FRAD033_EAC_00001"]),
                },
            ),
        ]
        expected = [ExtEntity(*vals) for vals in expected]
        fpath = self.datapath("FRAD033_EAC_00001_simplified.xml")
        import_log = SimpleImportLog(fpath)
        # Use a predictable extid_generator.
        extid_generator = map(str, count()).__next__
        importer = dataimport.EACCPFImporter(
            fpath, import_log, mock_, extid_generator=extid_generator
        )
        entities = list(importer.external_entities())
        # Used for an easier handling of the order error while generating the 2 lists
        self.check_order_entities(entities, expected)
        self.check_external_entities(entities, expected)
        visited = set([])
        for x in importer._visited.values():
            visited.update(x)
        self.assertCountEqual(visited, [x.extid for x in expected])
        # Gather not-visited tag by name and group source lines.
        not_visited = {}
        for tagname, sourceline in importer.not_visited():
            not_visited.setdefault(tagname, set([])).add(sourceline)
        self.assertEqual(
            not_visited,
            {
                "maintenanceStatus": set([12]),
                "publicationStatus": set([14]),
                "recordId": set([8]),
                "maintenanceAgency": set([16]),
                "languageDeclaration": set([21]),
                "languageUsed": set([188, 195]),
                "localControl": set([54]),
                "source": set([76]),  # empty.
                "structureOrGenealogy": set([268]),  # empty.
                "biogHist": set([328, 331]),  # empty.
            },
        )

    def check_order_entities(self, entities, expected):
        """Usefull test for comparing sorted lists of actual and
        expected entities. Make it easier to check where to add a
        new entity or swap 2 of them.
        """

        def get_sorted(elems):
            return sorted(
                ((e.etype, e.extid) for e in elems if e.etype != "ExternalUri"),
                key=lambda e: e[1],
            )

        a_lst = get_sorted(entities)
        e_lst = get_sorted(expected)
        self.assertEquals(a_lst, e_lst)

    def test_values_from_functions(self):
        fname = "FRAD033_EAC_00001_simplified.xml"
        fpath = self.datapath(fname)
        self.root = etree.fromstring(XML_TEST)
        import_log = SimpleImportLog(fpath)
        importer = dataimport.EACCPFImporter(fpath, import_log)
        values = importer.values_from_xpaths(
            self.root,
            (
                ("to_value", "to"),
                ("from_value", "from"),
                ("heading_value", "heading"),
                ("body_value", "body"),
                ("empty_value", "empty"),
            ),
        )
        self.assertEqual(
            values,
            {
                "to_value": set(["Tove"]),
                "from_value": set(["Jani"]),
                "heading_value": set(["Reminder"]),
                "body_value": set(["Hey!"]),
            },
        )
        attrib = importer.values_from_attrib(
            self.root, (("test_varname", "test"), ("test_varname_2", "test2"))
        )
        self.assertEqual(
            attrib,
            {
                "test_varname": set(["test_value"]),
                "test_varname_2": set(["test2_value"]),
            },
        )

    def test_mandate_under_mandates(self):
        """In FRAD033_EAC_00003.xml, <mandate> element are within <mandates>."""
        entities = list(self.file_extentities("FRAD033_EAC_00003.xml"))
        expected_terms = [
            "Code du patrimoine, Livre II",
            "Loi du 5 brumaire an V [26 octobre 1796]",
            (
                "Loi du 3 janvier 1979 sur les archives, accompagnée de ses décrets\n"
                "                        d’application datant du 3 décembre."
            ),
            "Loi sur les archives du 15 juillet 2008",
        ]
        self.assertCountEqual(
            [
                next(iter(x.values["term"]))
                for x in entities
                if x.etype == "Mandate" and "term" in x.values
            ],
            expected_terms,
        )
        mandate_with_link = next(
            x
            for x in entities
            if x.etype == "Mandate"
            and "Code du patrimoine, Livre II" in x.values["term"]
        )
        extid = next(iter(mandate_with_link.values["has_citation"]))
        url = (
            "http://www.legifrance.gouv.fr/affichCode.do?idArticle=LEGIARTI000019202816"
        )
        citation = next(
            x for x in entities if x.etype == "Citation" and url in x.values["uri"]
        )
        self.assertEqual(extid, citation.extid)

    def test_agentfunction_within_functions_tag(self):
        """In FRAD033_EAC_00003.xml, <function> element are within <functions>
        not <description>.
        """
        entities = self.file_extentities("FRAD033_EAC_00003.xml")
        self.assertCountEqual(
            [
                x.values["name"].pop()
                for x in entities
                if x.etype == "AgentFunction" and "name" in x.values
            ],
            ["contr\xf4le", "collecte", "classement", "restauration", "promotion"],
        )

    def test_no_nameentry_authorizedform(self):
        entities = self.file_extentities(
            "Service de l'administration generale et des assemblees.xml"
        )
        expected = (
            "Gironde. Conseil général. Service de l'administration "
            "générale et des assemblées"
        )
        self.assertIn(
            expected,
            [x.values["parts"].pop() for x in entities if x.etype == "NameEntry"],
        )

    def ctx_assert(self, method, actual, expected, ctx, msg=None):
        """Wrap assertion method with a context message"""
        try:
            getattr(self, method)(actual, expected, msg=msg)
        except AssertionError as exc:
            msg = str(exc)
            if ctx:
                msg = ("[%s] " % ctx) + msg
            raise AssertionError(msg)

    def check_external_entities(self, entities, expected):
        entities = extentities2dict(entities)
        expected = extentities2dict(expected)
        etypes, expected_etypes = list(entities), list(expected)
        self.ctx_assert("assertCountEqual", etypes, expected_etypes, ctx="etypes")

        def safe_int(value):
            try:
                return int(value)
            except ValueError:
                return 9999

        ordered_etypes = [
            x[1]
            for x in sorted(
                (min(safe_int(extid) for extid in edict), etype)
                for etype, edict in expected.items()
            )
        ]
        for etype in ordered_etypes:
            edict = expected[etype]
            entities_etype = entities[etype]
            extids, expected_extids = list(entities_etype), list(edict)
            self.ctx_assert(
                "assertCountEqual", extids, expected_extids, ctx="%s/extids" % etype
            )
            for extid, values in edict.items():
                self.ctx_assert(
                    "assertEqual",
                    tolist(entities_etype[extid]),
                    tolist(values),
                    ctx="%s/%s/values" % (etype, extid),
                )

    def test_errors(self):
        log = SimpleImportLog("<dummy>")
        with self.assertRaises(dataimport.InvalidXML):
            importer = dataimport.EACCPFImporter(BytesIO(b"no xml"), log, mock_)
            list(importer.external_entities())
        with self.assertRaises(dataimport.MissingTag):
            importer = dataimport.EACCPFImporter(BytesIO(b"<xml/>"), log, mock_)
            list(importer.external_entities())


class EACDataImportTC(CubicWebTC):
    def test_FRAD033_EAC_00001(self):
        fpath = self.datapath("FRAD033_EAC_00001_simplified.xml")
        with self.admin_access.repo_cnx() as cnx:
            self.maxDiff = None
            # create a skos concept to ensure it's used instead of a ExternalUri
            scheme = cnx.create_entity("ConceptScheme")
            scheme.add_concept(
                "environnement",
                cwuri="http://data.culture.fr/thesaurus/page/ark:/67717/T1-1074",
            )
            cnx.commit()
            created, updated = testutils.eac_import(cnx, fpath)
            self.assertEqual(len(created), 80)
            self.assertEqual(updated, set())
            rset = cnx.find("AuthorityRecord", isni="22330001300016")
            self.assertEqual(len(rset), 1)
            record = rset.one()
            self.assertEqual(record.kind, "authority")
            self.assertEqual(record.start_date, datetime.date(1800, 1, 1))
            self.assertEqual(record.end_date, datetime.date(2099, 1, 1))
            self.assertEqual(
                record.other_record_ids, [(None, "1234"), ("letters", "ABCD")]
            )
            address = record.postal_address[0]
            self.assertEqual(address.street, "1 Esplanade Charles de Gaulle")
            self.assertEqual(address.postalcode, "33074")
            self.assertEqual(address.city, " Bordeaux Cedex")
            self.assertEqual(
                address.raw_address,
                "1 Esplanade Charles de Gaulle\n33074\n Bordeaux Cedex",
            )
            rset = cnx.execute(
                """
                 Any R,N WHERE P place_agent A, A eid %(eid)s,
                 P role R, P place_entry_relation E, E name N""",
                {"eid": record.eid},
            )
            self.assertCountEqual(
                rset.rows,
                [
                    ["siege", "Bordeaux (Gironde, France)"],
                    ["domicile", "Toulouse (France)"],
                    ["dodo", "Lit"],
                ],
            )
            self.assertEqual(len(record.reverse_function_agent), 3)
            for related in (
                "structure",
                "history",
                "mandate",
                "occupation",
                "generalcontext",
                "legal_status",
                "eac_relations",
                "equivalent_concept",
                "control",
                "convention",
                "parallel_relations",
            ):
                with self.subTest(related=related):
                    checker = getattr(self, "_check_" + related)
                    checker(cnx, record)

    def _check_structure(self, cnx, record):
        rset = cnx.find("Structure", structure_agent=record)
        self.assertEqual(len(rset), 1)
        self.assertEqual(
            rset.one().printable_value("description", format="text/plain").strip(),
            "Pour accomplir ses missions ...",
        )

    def _check_convention(self, cnx, record):
        rset = cnx.find("Convention", convention_of=record).sorted_rset(lambda x: x.eid)
        self.assertEqual(len(rset), 3)
        self.assertEqual(
            rset.get_entity(0, 0)
            .printable_value("description", format="text/plain")
            .strip(),
            "Norme ISAAR(CPF) du Conseil international des archives, "
            "2e \xe9dition, 1996.",
        )

    def _check_history(self, cnx, record):
        rset = cnx.find("History", history_agent=record)
        self.assertEqual(len(rset), 1)
        entity = rset.one()
        self.assertEqual(
            entity.printable_value("abstract", format="text/plain").strip(),
            "Test of an abstract element",
        )
        self.assertEqual(
            entity.printable_value("text", format="text/plain").strip(),
            "La loi du 22 décembre 1789, en divisant ...\n\nL'inspecteur Canardo",
        )
        events = rset.one().has_event
        self.assertEqual(len(events), 2)

    def _check_mandate(self, cnx, record):
        rset = cnx.find("Mandate", mandate_agent=record)
        self.assertEqual(len(rset), 1)
        self.assertEqual(
            rset.one().printable_value("description", format="text/plain").strip(),
            "Description du mandat",
        )

    def _check_occupation(self, cnx, record):
        occupation = cnx.find("Occupation", occupation_agent=record).one()
        self.assertEqual(occupation.term, "Réunioniste")
        citation = occupation.has_citation[0]
        self.assertEqual(citation.note, "la bible")
        voc = occupation.equivalent_concept[0]
        self.assertEqual(voc.uri, "http://pifgadget.com")

    def _check_generalcontext(self, cnx, record):
        occupation = cnx.find("GeneralContext", general_context_of=record).one()
        self.assertIn("very famous", occupation.content)
        self.assertEqual(occupation.content_format, "text/html")
        citation = occupation.has_citation[0]
        self.assertEqual(citation.note, "it's well known")

    def _check_legal_status(self, cnx, record):
        rset = cnx.find("LegalStatus", legal_status_agent=record)
        self.assertEqual(len(rset), 1)
        self.assertEqual(
            rset.one().printable_value("description", format="text/plain").strip(),
            "Description du statut",
        )

    def _check_eac_relations(self, cnx, record):
        relation = cnx.find("HierarchicalRelation").one()
        self.assertEqual(
            relation.entry,
            "Gironde. Conseil général. Direction de "
            "l'administration et de la sécurité juridique",
        )
        self.assertEqual(
            relation.printable_value("description", format="text/plain"), "Coucou"
        )
        other_record = cnx.find("ExternalUri", uri="CG33-DIRADSJ").one()
        self.assertEqual(relation.hierarchical_parent[0], other_record)
        relation = cnx.find("AssociationRelation").one()
        self.assertEqual(relation.association_from[0], record)
        other_record = cnx.find("ExternalUri", uri="agent-x").one()
        self.assertEqual(other_record.cwuri, "agent-x")
        self.assertEqual(relation.association_to[0], other_record)
        rset = cnx.find("EACResourceRelation", agent_role="creatorOf")
        self.assertEqual(len(rset), 1)
        rrelation = rset.one()
        self.assertEqual(rrelation.resource_relation_agent[0], record)
        exturi = rrelation.resource_relation_resource[0]
        self.assertEqual(exturi.uri, "http://gael.gironde.fr/ead.html?id=FRAD033_IR_N")
        self.assertEqual(
            rrelation.xml_wrap.getvalue().decode(),
            '<he xmlns="urn:isbn:1-931666-33-4" xmlns:xlink="http'
            '://www.w3.org/1999/xlink" xmlns:xsi="http://www.w3.org'
            '/2001/XMLSchema-instance">joe</he>',
        )
        self.assertEqual(
            rrelation.json_attrs,
            {
                "{http://www.w3.org/1999/xlink}actuate": "onRequest",
                "{http://www.w3.org/1999/xlink}show": "new",
                "{http://www.w3.org/1999/xlink}type": "simple",
            },
        )
        rset = cnx.find("EACFunctionRelation", r_type="performs")
        func_relation = rset.one()
        self.assertEqual(
            func_relation.json_attrs,
            {
                "{http://www.w3.org/1999/xlink}actuate": "onLoad",
                "{http://www.w3.org/1999/xlink}arcrole": "http://test_arcrole.lol.com",
                "{http://www.w3.org/1999/xlink}role": "http://test_role.lmao.com",
            },
        )
        self.assertEqual(
            func_relation.relation_entry,
            "Alumni communication\n\tmanagement, " "University of\n\tGlasgow\n\t",
        )
        self.assertEqual(
            func_relation.xml_wrap.getvalue().decode(),
            '<mods xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"'
            ' xmlns="urn:isbn:1-931666-33-4" xmlns:xlink="http://www.w3'
            '.org/1999/xlink" xsi:schemaLocation="http://www.loc.gov'
            "/mods/v3 http:         //www.loc.gov/mods/v3/mods-3-3.xsd"
            '">\n\t    <titleInfo>\n\t      <title>Artisti trentini tra'
            " le due\n\t      guerre\n\t      </title>\n\t    </titleInfo>\n"
            '\t    <name>\n\t      <namePart type="given">Nicoletta\n\t'
            '      </namePart>\n\t      <namePart type="family">Boschiero\n'
            '\t      </namePart>\n\t      <role>\n\t\t<roleTerm type="text'
            '">autore\n\t\t</roleTerm>\n\t      </role>\n\t    </name>\n\t'
            "  </mods>\n\t",
        )
        self.assertEqual(func_relation.function_relation_agent[0], record)
        self.assertEqual(
            func_relation.function_relation_function[0].uri,
            "http://gael.gironde.fr/ead.html?id=FRAD033_IR_N",
        )
        rset = cnx.find("EACFunctionRelation", r_type="controls")
        func_relation = rset.one()
        self.assertEqual(func_relation.function_relation_agent[0], record)
        self.assertEqual(
            func_relation.function_relation_function[0].uri, "FRAD033_IR_N"
        )

    def _check_parallel_relations(self, cnx, record):
        rset = cnx.find("ParallelNames", parallel_names_of=record).sorted_rset(
            lambda x: x.eid
        )
        self.assertEqual(len(rset), 2)
        p_entity = rset.get_entity(0, 0)
        self.assertEqual(p_entity.parallel_names_of[0], record)
        self.assertEqual(len(p_entity.simple_name_relation), 2)
        self.assertEqual(len(p_entity.date_relation), 0)
        p_entity = rset.get_entity(1, 0)
        self.assertEqual(p_entity.parallel_names_of[0], record)
        self.assertEqual(len(p_entity.simple_name_relation), 3)
        self.assertEqual(len(p_entity.date_relation), 3)

    def _check_equivalent_concept(self, cnx, record):
        functions = dict((f.name, f) for f in record.reverse_function_agent)
        self.assertEqual(
            functions["action sociale"].equivalent_concept[0].cwuri,
            "http://data.culture.fr/thesaurus/page/ark:/67717/T1-200",
        )
        self.assertEqual(
            functions["action sociale"].equivalent_concept[0].cw_etype, "ExternalUri"
        )
        self.assertEqual(
            functions["environnement"].equivalent_concept[0].cwuri,
            "http://data.culture.fr/thesaurus/page/ark:/67717/T1-1074",
        )
        self.assertEqual(
            functions["environnement"].equivalent_concept[0].cw_etype, "Concept"
        )
        self.assertEqual(
            functions["environnement"].vocabulary_source[0].eid,
            functions["environnement"].equivalent_concept[0].scheme.eid,
        )
        place = cnx.find("PlaceEntry", name="Bordeaux (Gironde, France)").one()
        self.assertEqual(
            place.equivalent_concept[0].cwuri,
            "http://catalogue.bnf.fr/ark:/12148/cb152418385",
        )

    def _check_control(self, cnx, record):
        rset = cnx.find("EACSource")
        self.assertEqual(len(rset), 2)
        rset = cnx.execute("Any A WHERE A generated X, X eid %(x)s", {"x": record.eid})
        self.assertEqual(len(rset), 2)
        rset = cnx.execute('Any A WHERE A agent "Delphine Jamet"')
        self.assertEqual(len(rset), 1)

    def test_multiple_imports(self):
        def count_entity(cnx, etype):
            return cnx.execute("Any COUNT(X) WHERE X is %s" % etype)[0][0]

        with self.admin_access.repo_cnx() as cnx:
            nb_records_before = count_entity(cnx, "AuthorityRecord")
            for fname in (
                "FRAD033_EAC_00001.xml",
                "FRAD033_EAC_00003.xml",
                "FRAD033_EAC_00071.xml",
            ):
                fpath = self.datapath(fname)
                created, updated = testutils.eac_import(cnx, fpath)
            nb_records_after = count_entity(cnx, "AuthorityRecord")
            self.assertEqual(nb_records_after - nb_records_before, 3)

    def test_unknown_kind(self):
        with self.admin_access.repo_cnx() as cnx:
            testutils.eac_import(cnx, self.datapath("custom_kind.xml"))
            self.assertRaises(
                NoResultError, cnx.find("AgentKind", name="a custom kind").one
            )
            self.assertEqual(
                cnx.find("AuthorityRecord").one().agent_kind[0].name,
                "unknown-agent-kind",
            )

    def test_no_name_entry(self):
        with self.admin_access.repo_cnx() as cnx:
            with self.assertRaises(dataimport.MissingTag) as cm:
                testutils.eac_import(cnx, self.datapath("no_name_entry.xml"))
            self.assertEqual(cm.exception.tag, "nameEntry")
            self.assertEqual(cm.exception.tag_parent, "identity")

    def test_no_name_entry_part(self):
        with self.admin_access.repo_cnx() as cnx:
            with self.assertRaises(dataimport.MissingTag) as cm:
                testutils.eac_import(cnx, self.datapath("no_name_entry_part.xml"))
            self.assertEqual(cm.exception.tag, "part")
            self.assertEqual(cm.exception.tag_parent, "nameEntry")


if __name__ == "__main__":
    unittest.main()
